@@ -38,11 +38,29 @@
 #include <stdio.h>
 #include <string.h>
 
+#ifndef TRUE
+#define TRUE    1
+#endif
+
+#ifndef FALSE
+#define FALSE   0
+#endif
+
+
+struct ipmi_intf;
+
 struct valstr {
 	uint16_t val;
 	const char * str;
 };
+struct oemvalstr {
+	uint16_t oem;
+   uint16_t val;
+	const char * str;
+};
+
 const char * val2str(uint16_t val, const struct valstr * vs);
+const char * oemval2str(uint16_t oem,uint16_t val, const struct oemvalstr * vs);
 uint16_t str2val(const char * str, const struct valstr * vs);
 void print_valstr(const struct valstr * vs, const char * title, int loglevel);
 void print_valstr_2col(const struct valstr * vs, const char * title, int loglevel);
@@ -54,7 +72,7 @@ const char * buf2str(uint8_t * buf, int len);
 void printbuf(const uint8_t * buf, int len, const char * desc);
 uint8_t ipmi_csum(uint8_t * d, int s);
 FILE * ipmi_open_file(const char * file, int rw);
-void ipmi_start_daemon(void);
+void ipmi_start_daemon(struct ipmi_intf *intf);
 
 #define ipmi_open_file_read(file)	ipmi_open_file(file, 0)
 #define ipmi_open_file_write(file)	ipmi_open_file(file, 1)
