@@ -159,7 +159,7 @@ struct sdr_record_mask {
 		struct {
 			uint16_t assert_event;	/* assertion event mask */
 			uint16_t deassert_event;	/* de-assertion event mask */
-			uint16_t read;	/* discrete reaading mask */
+			uint16_t read;	/* discrete reading mask */
 		} discrete;
 		struct {
 #if WORDS_BIGENDIAN
@@ -232,44 +232,52 @@ struct sdr_record_mask {
 			uint16_t status_unr:1;
 			uint16_t reserved_2:1;
 #endif
-			struct {
-#if WORDS_BIGENDIAN		/* settable threshold mask */
-				uint8_t reserved:2;
-				uint8_t unr:1;
-				uint8_t ucr:1;
-				uint8_t unc:1;
-				uint8_t lnr:1;
-				uint8_t lcr:1;
-				uint8_t lnc:1;
+			union {
+				struct {
+#if WORDS_BIGENDIAN			/* settable threshold mask */
+					uint16_t reserved:2;
+					uint16_t unr:1;
+					uint16_t ucr:1;
+					uint16_t unc:1;
+					uint16_t lnr:1;
+					uint16_t lcr:1;
+					uint16_t lnc:1;
+					/* padding lower 8 bits */
+					uint16_t readable:8;
 #else
-				uint8_t lnc:1;
-				uint8_t lcr:1;
-				uint8_t lnr:1;
-				uint8_t unc:1;
-				uint8_t ucr:1;
-				uint8_t unr:1;
-				uint8_t reserved:2;
+					uint16_t readable:8;
+					uint16_t lnc:1;
+					uint16_t lcr:1;
+					uint16_t lnr:1;
+					uint16_t unc:1;
+					uint16_t ucr:1;
+					uint16_t unr:1;
+					uint16_t reserved:2;
 #endif
-			} set;
-			struct {
-#if WORDS_BIGENDIAN		/* readable threshold mask */
-				uint8_t reserved:2;
-				uint8_t unr:1;
-				uint8_t ucr:1;
-				uint8_t unc:1;
-				uint8_t lnr:1;
-				uint8_t lcr:1;
-				uint8_t lnc:1;
+				} set;
+				struct {
+#if WORDS_BIGENDIAN			/* readable threshold mask */
+					/* padding upper 8 bits */
+					uint16_t settable:8;
+					uint16_t reserved:2;
+					uint16_t unr:1;
+					uint16_t ucr:1;
+					uint16_t unc:1;
+					uint16_t lnr:1;
+					uint16_t lcr:1;
+					uint16_t lnc:1;
 #else
-				uint8_t lnc:1;
-				uint8_t lcr:1;
-				uint8_t lnr:1;
-				uint8_t unc:1;
-				uint8_t ucr:1;
-				uint8_t unr:1;
-				uint8_t reserved:2;
+					uint16_t lnc:1;
+					uint16_t lcr:1;
+					uint16_t lnr:1;
+					uint16_t unc:1;
+					uint16_t ucr:1;
+					uint16_t unr:1;
+					uint16_t reserved:2;
+					uint16_t settable:8;
 #endif
-			} read;
+				} read;
+			};
 		} threshold;
 	} type;
 } __attribute__ ((packed));
@@ -763,8 +771,9 @@ static const char *unit_desc[] __attribute__ ((unused)) = {
 	    "overflow", "underrun", "collision", "packets", "messages",
 	    "characters", "error", "correctable error", "uncorrectable error",};
 
-/* sensor type codes (IPMI v1.5 table 36.3) */
-#define SENSOR_TYPE_MAX 0x29
+/* sensor type codes (IPMI v1.5 table 36.3) 
+  / Updated to v2.0 Table 42-3, Sensor Type Codes */
+#define SENSOR_TYPE_MAX 0x2C
 static const char *sensor_type_desc[] __attribute__ ((unused)) = {
 "reserved",
 	    "Temperature", "Voltage", "Current", "Fan",
@@ -779,7 +788,8 @@ static const char *sensor_type_desc[] __attribute__ ((unused)) = {
 	    "OS Boot", "OS Critical Stop", "Slot / Connector",
 	    "System ACPI Power State", "Watchdog", "Platform Alert",
 	    "Entity Presence", "Monitor ASIC", "LAN",
-	    "Management Subsystem Health", "Battery"};
+	    "Management Subsystem Health", "Battery","Session Audit",
+       "Version Change","FRU State" };
 
 struct ipmi_sdr_iterator *ipmi_sdr_start(struct ipmi_intf *intf);
 struct sdr_get_rs *ipmi_sdr_get_next_header(struct ipmi_intf *intf,
@@ -799,6 +809,8 @@ double sdr_convert_sensor_tolerance(struct sdr_record_full_sensor *sensor,
 				  uint8_t val);
 double sdr_convert_sensor_reading(struct sdr_record_full_sensor *sensor,
 				  uint8_t val);
+double sdr_convert_sensor_hysterisis(struct sdr_record_full_sensor *sensor,
+				  uint8_t val);
 uint8_t sdr_convert_sensor_value_to_raw(struct sdr_record_full_sensor *sensor,
 					double val);
 struct ipmi_rs *ipmi_sdr_get_sensor_reading(struct ipmi_intf *intf,
