@@ -54,6 +54,8 @@
 # define _PATH_VARRUN "/var/run/"
 #endif
 
+#include <ipmitool/ipmi.h>
+#include <ipmitool/ipmi_intf.h>
 #include <ipmitool/helper.h>
 #include <ipmitool/log.h>
 
@@ -97,13 +99,13 @@ void printbuf(const uint8_t * buf, int len, const char * desc)
 	if (verbose < 1)
 		return;
 
-	fprintf(stderr, "%s (%d bytes)\n", desc, len);
+	fprintf(stderr, "%s (%d bytes)\r\n", desc, len);
 	for (i=0; i<len; i++) {
 		if (((i%16) == 0) && (i != 0))
-			fprintf(stderr, "\n");
+			fprintf(stderr, "\r\n");
 		fprintf(stderr, " %2.2x", buf[i]);
 	}
-	fprintf(stderr, "\n");
+	fprintf(stderr, "\r\n");
 }
 
 const char * val2str(uint16_t val, const struct valstr *vs)
@@ -122,6 +124,35 @@ const char * val2str(uint16_t val, const struct valstr *vs)
 	return un_str;
 }
 
+const char * oemval2str(uint16_t oem, uint16_t val,
+                                             const struct oemvalstr *vs)
+{
+	static char un_str[32];
+	int i;
+
+	for (i = 0; vs[i].oem != 0x00 &&  vs[i].str != NULL; i++) {
+      /* FIXME: for now on we assume PICMG capability on all IANAs */
+      if
+      ( 
+         (
+            vs[i].oem == oem 
+            ||
+            vs[i].oem == IPMI_OEM_PICMG 
+         )
+         &&  
+         vs[i].val == val  
+      )
+      {
+         return vs[i].str;
+      }
+	}
+
+	memset(un_str, 0, 32);
+	snprintf(un_str, 32, "OEM reserved #%02x", val);
+
+	return un_str;
+}
+
 uint16_t str2val(const char *str, const struct valstr *vs)
 {
 	int i;
@@ -344,7 +375,7 @@ ipmi_open_file(const char * file, int rw)
 }
 
 void
-ipmi_start_daemon(void)
+ipmi_start_daemon(struct ipmi_intf *intf)
 {
 	pid_t pid;
 	int fd;
@@ -388,7 +419,7 @@ ipmi_start_daemon(void)
 		close(fd);
 	}
 #else
-	if (setpgrp() == -1)
+	if (setpgid(0, 0) == -1)
 		exit(1);
 	pid = (pid_t) fork();
 	if (pid < 0 || pid > 0)
@@ -398,11 +429,12 @@ ipmi_start_daemon(void)
 	chdir("/");
 	umask(0);
 
-	for (fd=0; fd<64; fd++)
-		close(fd);
+	for (fd=0; fd<64; fd++) {
+		if (fd != intf->fd)
+			close(fd);
+	}
 
 	open("/dev/null", O_RDWR);
 	dup(0);
 	dup(0);
 }
-
