@@ -28,10 +28,6 @@
  * PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
  * LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE,
  * EVEN IF SUN HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
- * 
- * You acknowledge that this software is not designed or intended for use
- * in the design, construction, operation or maintenance of any nuclear
- * facility.
  */
 
 #include <stdlib.h>
@@ -113,8 +109,10 @@ ipmi_chassis_power_control(struct ipmi_intf * intf, uint8_t ctl)
 	printf("Chassis Power Control: %s\n",
 	       val2str(ctl, ipmi_chassis_power_control_vals));
 
+#if 0	/* this can cause sessions to hang around after power commands */
 	/* sessions often get lost when changing chassis power */
 	intf->abort = 1;
+#endif
 
 	return 0;
 }
@@ -174,7 +172,7 @@ ipmi_chassis_identify(struct ipmi_intf * intf, char * arg)
 		printf("default (15 seconds)\n");
 	} else {
 		if (identify_data.force_on != 0) {
-			printf("indefinate\n");
+			printf("indefinite\n");
 		} else {
 			if (identify_data.interval == 0)
 				printf("off\n");
@@ -358,6 +356,75 @@ ipmi_chassis_status(struct ipmi_intf * intf)
 	return 0;
 }
 
+
+static int
+ipmi_chassis_selftest(struct ipmi_intf * intf)
+{
+	struct ipmi_rs * rsp;
+	struct ipmi_rq req;
+
+	memset(&req, 0, sizeof(req));
+	req.msg.netfn = IPMI_NETFN_APP;
+	req.msg.cmd = 0x4;
+
+	rsp = intf->sendrecv(intf, &req);
+	if (rsp == NULL) {
+		lprintf(LOG_ERR, "Error sending Get Self Test command");
+		return -1;
+	}
+	if (rsp->ccode > 0) {
+		lprintf(LOG_ERR, "Error sending Get Self Test command: %s",
+			val2str(rsp->ccode, completion_code_vals));
+		return -1;
+	}
+
+	printf("Self Test Results    : ");
+	switch (rsp->data[0]) {
+	case 0x55:
+		printf("passed\n");
+		break;
+
+	case 0x56:
+		printf("not implemented\n");
+		break;
+
+	case 0x57:
+	{
+		int i;
+		const struct valstr broken_dev_vals[] = {
+			{ 0, "firmware corrupted" },
+			{ 1, "boot block corrupted" },
+			{ 2, "FRU Internal Use Area corrupted" },
+			{ 3, "SDR Repository empty" },
+			{ 4, "IPMB not responding" },
+			{ 5, "cannot access BMC FRU" },
+			{ 6, "cannot access SDR Repository" },
+			{ 7, "cannot access SEL Device" },
+			{ 0xff, NULL },
+		};
+		printf("device error\n");
+		for (i=0; i<8; i++) {
+			if (rsp->data[1] & (1<<i)) {
+				printf("                       [%s]\n",
+				       val2str(i, broken_dev_vals));
+			}
+		}
+	}
+	break;
+
+	case 0x58:
+		printf("Fatal hardware error: %02xh\n", rsp->data[1]);
+		break;
+
+	default:
+		printf("Device-specific failure %02xh:%02xh\n",
+		       rsp->data[0], rsp->data[1]);
+		break;
+	}
+
+	return 0;
+}
+
 static int
 ipmi_chassis_set_bootparam(struct ipmi_intf * intf, uint8_t param, uint8_t * data, int len)
 {
@@ -381,8 +448,10 @@ ipmi_chassis_set_bootparam(struct ipmi_intf * intf, uint8_t param, uint8_t * dat
 		return -1;
 	}
 	if (rsp->ccode > 0) {
-		lprintf(LOG_ERR, "Set Chassis Boot Parameter %d failed: %s",
-			param, val2str(rsp->ccode, completion_code_vals));
+		if (param != 0) {
+			lprintf(LOG_ERR, "Set Chassis Boot Parameter %d failed: %s",
+				param, val2str(rsp->ccode, completion_code_vals));
+		}
 		return -1;
 	}
 
@@ -430,22 +499,49 @@ ipmi_chassis_get_bootparam(struct ipmi_intf * intf, char * arg)
 	printf("Boot parameter %d is %s\n", rsp->data[1] & 0x7f,
 	       (rsp->data[1] & 0x80) ? "invalid/locked" : "valid/unlocked");
 	printf("Boot parameter data: %s\n", buf2str(rsp->data+2, rsp->data_len - 2));
+
+	return 0;
 }
 
 static int
-ipmi_chassis_set_bootdev(struct ipmi_intf * intf, char * arg, int clearcmos)
+ipmi_chassis_set_bootdev(struct ipmi_intf * intf, char * arg, uint8_t *iflags)
 {
 	uint8_t flags[5];
 	int rc = 0;
+	int use_progress = 1;
+
+	if (use_progress) {
+		/* set set-in-progress flag */
+		memset(flags, 0, 5);
+		flags[0] = 0x01;
+		rc = ipmi_chassis_set_bootparam(intf,
+			IPMI_CHASSIS_BOOTPARAM_SET_IN_PROGRESS, flags, 1);
+		if (rc < 0)
+			use_progress = 0;
+	}
 
 	memset(flags, 0, 5);
 	flags[0] = 0x01;
 	flags[1] = 0x01;
-	rc = ipmi_chassis_set_bootparam(intf, 4, flags, 2);
-	if (rc < 0)
+	rc = ipmi_chassis_set_bootparam(intf, IPMI_CHASSIS_BOOTPARAM_INFO_ACK,
+					flags, 2);
+
+	if (rc < 0) {
+		if (use_progress) {
+			/* set-in-progress = set-complete */
+			memset(flags, 0, 5);
+			ipmi_chassis_set_bootparam(intf,
+				   IPMI_CHASSIS_BOOTPARAM_SET_IN_PROGRESS,
+				   flags, 1);
+		}
 		return -1;
+	}
+
+	if (iflags == NULL)
+			memset(flags, 0, 5);
+	else
+			memcpy(flags, iflags, sizeof (flags));
 
-	memset(flags, 0, 5);
 	if (arg == NULL)
 		flags[1] = 0x00;
 	else if (strncmp(arg, "none", 4) == 0)
@@ -473,17 +569,42 @@ ipmi_chassis_set_bootdev(struct ipmi_intf * intf, char * arg, int clearcmos)
 		flags[1] = 0x18;
 	else {
 		lprintf(LOG_ERR, "Invalid argument: %s", arg);
+		if (use_progress) {
+			/* set-in-progress = set-complete */
+			memset(flags, 0, 5);
+			ipmi_chassis_set_bootparam(intf,
+				   IPMI_CHASSIS_BOOTPARAM_SET_IN_PROGRESS,
+				   flags, 1);
+		}
 		return -1;
 	}
 
-	if (clearcmos)
-		flags[1] |= 0x80;
-
 	/* set flag valid bit */
-	flags[0] = 0x80;
-	rc = ipmi_chassis_set_bootparam(intf, 5, flags, 5);
-	if (rc == 0)
+	flags[0] |= 0x80;
+
+	rc = ipmi_chassis_set_bootparam(intf, IPMI_CHASSIS_BOOTPARAM_BOOT_FLAGS,
+					flags, 5);
+	if (rc == 0) {
+		if (use_progress) {
+			/* set-in-progress = commit-write */
+			memset(flags, 0, 5);
+			flags[0] = 0x02;
+			ipmi_chassis_set_bootparam(intf,
+				   IPMI_CHASSIS_BOOTPARAM_SET_IN_PROGRESS,
+				   flags, 1);
+		}
+
 		printf("Set Boot Device to %s\n", arg);
+	}
+
+	if (use_progress) {
+		/* set-in-progress = set-complete */
+		memset(flags, 0, 5);
+		ipmi_chassis_set_bootparam(intf,
+			   IPMI_CHASSIS_BOOTPARAM_SET_IN_PROGRESS,
+			   flags, 1);
+	}
+
 	return rc;
 }
 
@@ -539,17 +660,55 @@ ipmi_chassis_power_policy(struct ipmi_intf * intf, uint8_t policy)
 	return 0;
 }
 
+int
+ipmi_power_main(struct ipmi_intf * intf, int argc, char ** argv)
+{
+	int rc = 0;
+	uint8_t ctl = 0;
+
+	if ((argc < 1) || (strncmp(argv[0], "help", 4) == 0)) {
+		lprintf(LOG_NOTICE, "chassis power Commands: status, on, off, cycle, reset, diag, soft");
+		return 0;
+	}
+	if (strncmp(argv[0], "status", 6) == 0) {
+		rc = ipmi_chassis_print_power_status(intf);
+		return rc;
+	}
+	if ((strncmp(argv[0], "up", 2) == 0) || (strncmp(argv[0], "on", 2) == 0))
+		ctl = IPMI_CHASSIS_CTL_POWER_UP;
+	else if ((strncmp(argv[0], "down", 4) == 0) || (strncmp(argv[0], "off", 3) == 0))
+		ctl = IPMI_CHASSIS_CTL_POWER_DOWN;
+	else if (strncmp(argv[0], "cycle", 5) == 0)
+		ctl = IPMI_CHASSIS_CTL_POWER_CYCLE;
+	else if (strncmp(argv[0], "reset", 5) == 0)
+		ctl = IPMI_CHASSIS_CTL_HARD_RESET;
+	else if (strncmp(argv[0], "diag", 4) == 0)
+		ctl = IPMI_CHASSIS_CTL_PULSE_DIAG;
+	else if ((strncmp(argv[0], "acpi", 4) == 0) || (strncmp(argv[0], "soft", 4) == 0))
+		ctl = IPMI_CHASSIS_CTL_ACPI_SOFT;
+	else {
+		lprintf(LOG_ERR, "Invalid chassis power command: %s", argv[0]);
+		return -1;
+	}
+
+	rc = ipmi_chassis_power_control(intf, ctl);
+	return rc;
+}
+
 int
 ipmi_chassis_main(struct ipmi_intf * intf, int argc, char ** argv)
 {
 	int rc = 0;
 
 	if ((argc == 0) || (strncmp(argv[0], "help", 4) == 0)) {
-		lprintf(LOG_NOTICE, "Chassis Commands:  status, power, identify, policy, restart_cause, poh, bootdev");
+		lprintf(LOG_NOTICE, "Chassis Commands:  status, power, identify, policy, restart_cause, poh, bootdev, bootparam, selftest");
 	}
 	else if (strncmp(argv[0], "status", 6) == 0) {
 		rc = ipmi_chassis_status(intf);
 	}
+	else if (strncmp(argv[0], "selftest", 8) == 0) {
+		rc = ipmi_chassis_selftest(intf);
+	}
 	else if (strncmp(argv[0], "power", 5) == 0) {
 		uint8_t ctl = 0;
 
@@ -643,7 +802,7 @@ ipmi_chassis_main(struct ipmi_intf * intf, int argc, char ** argv)
 					lprintf(LOG_NOTICE, "bootparam set <option> [value ...]");
 				} else {
 					if (strncmp(argv[2], "bootflag", 8) == 0)
-						rc = ipmi_chassis_set_bootdev(intf, argv[3], 0);
+						rc = ipmi_chassis_set_bootdev(intf, argv[3], NULL);
 					else
 						lprintf(LOG_NOTICE, "bootparam set <option> [value ...]");
 				}
@@ -655,6 +814,7 @@ ipmi_chassis_main(struct ipmi_intf * intf, int argc, char ** argv)
 	else if (strncmp(argv[0], "bootdev", 7) == 0) {
 		if ((argc < 2) || (strncmp(argv[1], "help", 4) == 0)) {
 			lprintf(LOG_NOTICE, "bootdev <device> [clear-cmos=yes|no]");
+			lprintf(LOG_NOTICE, "bootdev <device> [options=help,...]");
 			lprintf(LOG_NOTICE, "  none  : Do not change boot device order");
 			lprintf(LOG_NOTICE, "  pxe   : Force PXE boot");
 			lprintf(LOG_NOTICE, "  disk  : Force boot from default Hard-drive");
@@ -662,17 +822,116 @@ ipmi_chassis_main(struct ipmi_intf * intf, int argc, char ** argv)
 			lprintf(LOG_NOTICE, "  diag  : Force boot from Diagnostic Partition");
 			lprintf(LOG_NOTICE, "  cdrom : Force boot from CD/DVD");
 			lprintf(LOG_NOTICE, "  bios  : Force boot into BIOS Setup");
+			lprintf(LOG_NOTICE, "  floppy: Force boot from Floppy/primary removable media");
 		} else {
 			if (argc < 3)
-				rc = ipmi_chassis_set_bootdev(intf, argv[1], 0);
+				rc = ipmi_chassis_set_bootdev(intf, argv[1], NULL);
 			else if (strncmp(argv[2], "clear-cmos=", 11) == 0) {
-				if (strncmp(argv[2]+11, "yes", 3) == 0)
-					rc = ipmi_chassis_set_bootdev(intf, argv[1], 1);
-				else 
-					rc = ipmi_chassis_set_bootdev(intf, argv[1], 0);
+				if (strncmp(argv[2]+11, "yes", 3) == 0) {
+					uint8_t flags[5] = {0, (1<<7), 0, 0, 0};
+
+					rc = ipmi_chassis_set_bootdev(intf, argv[1], flags);
+				} else 
+					rc = ipmi_chassis_set_bootdev(intf, argv[1], NULL);
+			}
+			else if (strncmp(argv[2], "options=", 8) == 0) {
+				char *token;
+				char *saveptr = NULL;
+				int optionError = 0;
+				unsigned char flags[5];
+				static struct {
+					char *name;
+					int i;
+					unsigned char mask;
+					unsigned char value;
+					char *desc;
+				} options[] = {
+					/* data 1 */
+					{"valid", 0, (1<<7), (1<<7),
+					 "Boot flags valid"},
+					{"persistent", 0, (1<<6), (1<<6),
+					 "Changes are persistent for all future boots"},
+					{"efiboot", 0, (1<<5), (1<<5),
+					 "Extensible Firmware Interface Boot (EFI)"},
+
+					/* data 2 */
+					{"clear-cmos", 1, (1<<7), (1<<7),
+					 "CMOS clear"},
+					{"lockkbd", 1, (1<<6), (1<<6),
+					 "Lock Keyboard"},
+					/* data2[5:2] is parsed elsewhere */
+					{"screenblank", 1, (1<<1), (1<<1),
+					 "Screen Blank"},
+					{"lockoutreset", 1, (1<<0), (1<<0),
+					 "Lock out Resetbuttons"},
+
+					/* data 3 */
+					{"lockout_power", 2, (1<<7), (1<<7),
+					 "Lock out (power off/sleep request) via Power Button"},
+					{"verbose=default", 2, (3<<5), (0<<5),
+					 "Request quiet BIOS display"},
+					{"verbose=no", 2, (3<<5), (1<<5),
+					 "Request quiet BIOS display"},
+					{"verbose=yes", 2, (3<<5), (2<<5),
+					 "Request verbose BIOS display"},
+					{"force_pet", 2, (1<<4), (1<<4),
+					 "Force progress event traps"},
+					{"upw_bypass", 2, (1<<3), (1<<3),
+					 "User password bypass"},
+					{"lockout_sleep", 2, (1<<2), (1<<2),
+					 "Log Out Sleep Button"},
+					{"cons_redirect=default", 2, (3<<0), (0<<0),
+					 "Console redirection occurs per BIOS configuration setting"},
+					{"cons_redirect=skip", 2, (3<<0), (1<<0),
+					 "Suppress (skip) console redirection if enabled"},
+					{"cons_redirect=enable", 2, (3<<0), (2<<0),
+					 "Suppress (skip) console redirection if enabled"},
+
+					/* data 4 */
+					/* data4[7:4] reserved */
+					/* data4[3] BIOS Shared Mode Override, not implemented here */
+					/* data4[2:0] BIOS Mux Control Override, not implemented here */
+
+					/* data5 reserved */
+					
+					{NULL}	/* End marker */
+				}, *op;
+
+				memset(&flags[0], 0, sizeof(flags));
+				token = strtok_r(argv[2] + 8, ",", &saveptr);
+				while (token != NULL) {
+					int i;
+
+					if (strcmp(token, "help") == 0) {
+						optionError = 1;
+						break;
+					}
+					for (op = options; op->name != NULL; ++op) {
+						if (strcmp(token, op->name) == 0) {
+								flags[op->i] &= op->mask;
+								flags[op->i] |= op->value;
+								break;
+						}
+					}
+					if (op->name == NULL) {
+						/* Option not found */
+						optionError = 1;
+						lprintf(LOG_ERR, "Invalid option: %s", token);
+					}
+					token = strtok_r(NULL, ",", &saveptr);
+				}
+				if (optionError) {
+						lprintf(LOG_NOTICE, "Legal options are:");
+						lprintf(LOG_NOTICE, "\thelp:\tprint this message");
+						for (op = options; op->name != NULL; ++op) {
+								lprintf(LOG_NOTICE, "\t%s:\t%s", op->name, op->desc);
+						}
+						return (-1);
+				}
+				rc = ipmi_chassis_set_bootdev(intf, argv[1], flags);
 			}
 			else
-				rc = ipmi_chassis_set_bootdev(intf, argv[1], 0);
+				rc = ipmi_chassis_set_bootdev(intf, argv[1], NULL);
 		}
 	}
 	else {
