@@ -37,17 +37,26 @@
 #include <string.h>
 #include <math.h>
 #include <time.h>
+#include <unistd.h>
 
 #include <ipmitool/helper.h>
 #include <ipmitool/ipmi.h>
 #include <ipmitool/ipmi_fwum.h>
 #include <ipmitool/ipmi_intf.h>
-
 #include <ipmitool/ipmi_mc.h>
 
+/******************************************************************************
+* HISTORY
+* ===========================================================================
+* 2007-01-11 [FI]
+*  - Incremented to version 1.3
+*  - Added lan packet size reduction mechanism to workaround fact
+*    that lan iface will not return C7 on excessive length
+*
+*****************************************************************************/
 
 #define VERSION_MAJ        1
-#define VERSION_MIN        1
+#define VERSION_MIN        3
 
 
 typedef enum eKFWUM_Task
@@ -123,17 +132,16 @@ typedef struct sKFWUM_SaveFirmwareInfo
    unsigned char       overheadSize;	
 }tKFWUM_SaveFirmwareInfo;	
   
-#define KFWUM_SMALL_BUFFER     28
-#define KFWUM_BIG_BUFFER       36
-#define KFWUM_OLD_CMD_OVERHEAD 6
-#define KFWUM_NEW_CMD_OVERHEAD 4
+#define KFWUM_SMALL_BUFFER     32 /* Minimum size (IPMB/IOL/old protocol) */
+#define KFWUM_BIG_BUFFER       32 /* Maximum size on KCS interface        */
+
+#define KFWUM_OLD_CMD_OVERHEAD 6  /*3 address + 1 size + 1 checksum + 1 command*/
+#define KFWUM_NEW_CMD_OVERHEAD 4  /*1 sequence+ 1 size + 1 checksum + 1 command*/
 #define KFWUM_PAGE_SIZE        256
 
 extern int verbose;
 static unsigned char fileName[512];
 static unsigned char firmBuf[1024*512];
-static unsigned char firmMaj;
-static unsigned char firmMinSub;
 static tKFWUM_SaveFirmwareInfo saveFirmwareInfo;
 
 static void KfwumOutputHelp(void);
@@ -157,7 +165,8 @@ static tKFWUM_Status KfwumManualRollback(struct ipmi_intf * intf);
 static tKFWUM_Status KfwumStartFirmwareImage(struct ipmi_intf * intf,
                                    unsigned long length,unsigned short padding);
 static tKFWUM_Status KfwumSaveFirmwareImage(struct ipmi_intf * intf,
-     unsigned char sequenceNumber, unsigned long address, unsigned char *pFirmBuf, unsigned char inBufLength);
+     unsigned char sequenceNumber, unsigned long address, 
+     unsigned char *pFirmBuf, unsigned char * pInBufLength);
 static tKFWUM_Status KfwumFinishFirmwareImage(struct ipmi_intf * intf,
                                                 tKFWUM_InFirmwareInfo firmInfo);
 static tKFWUM_Status KfwumUploadFirmware(struct ipmi_intf * intf,
@@ -172,9 +181,19 @@ static tKFWUM_Status KfwumGetTraceLog(struct ipmi_intf * intf);
 
 tKFWUM_Status KfwumValidFirmwareForBoard(tKFWUM_BoardInfo boardInfo,
                                                 tKFWUM_InFirmwareInfo firmInfo);
-tKFWUM_Status KfwumOutputInfo(tKFWUM_BoardInfo boardInfo,
+static void KfwumOutputInfo(tKFWUM_BoardInfo boardInfo,
                                                 tKFWUM_InFirmwareInfo firmInfo);
 
+
+/* ipmi_fwum_main  -  entry point for this ipmitool mode
+ *
+ * @intf:	  ipmi interface
+ * @arc     : number of arguments
+ * @argv    : point to argument array
+ *
+ * returns 0 on success
+ * returns -1 on error
+ */
 int ipmi_fwum_main(struct ipmi_intf * intf, int argc, char ** argv)
 {
    printf("FWUM extension Version %d.%d\n", VERSION_MAJ, VERSION_MIN);
@@ -203,7 +222,7 @@ int ipmi_fwum_main(struct ipmi_intf * intf, int argc, char ** argv)
             /* There is a file name in the parameters */
             if(strlen(argv[1]) < 512)
             {
-               strcpy(fileName, argv[1]);
+               strcpy((char *)fileName, argv[1]);
                printf("Firmware File Name         : %s\n", fileName);
 
                KfwumMain(intf, KFWUM_TASK_DOWNLOAD);
@@ -225,7 +244,7 @@ int ipmi_fwum_main(struct ipmi_intf * intf, int argc, char ** argv)
             /* There is a file name in the parameters */
             if(strlen(argv[1]) < 512)
             {
-               strcpy(fileName, argv[1]);
+               strcpy((char *)fileName, argv[1]);
                printf("Upgrading using file name %s\n", fileName);
                KfwumMain(intf, KFWUM_TASK_UPGRADE);
             }
@@ -314,8 +333,8 @@ static void KfwumMain(struct ipmi_intf * intf, tKFWUM_Task task)
 {
    tKFWUM_Status status = KFWUM_STATUS_OK;
    tKFWUM_BoardInfo boardInfo;
-   tKFWUM_InFirmwareInfo firmInfo;
-   unsigned long fileSize;
+   tKFWUM_InFirmwareInfo firmInfo = { 0 };
+   unsigned long fileSize = 0;
    static unsigned short padding;
 
    if((status == KFWUM_STATUS_OK) && (task == KFWUM_TASK_INFO))
@@ -443,13 +462,20 @@ static void KfwumMain(struct ipmi_intf * intf, tKFWUM_Task task)
 
 }
 
+/* KfwumGetFileSize  -  gets the file size
+ *
+ * @pFileName : filename ptr
+ * @pFileSize : output ptr for filesize
+ *
+ * returns KFWUM_STATUS_OK or KFWUM_STATUS_ERROR
+ */
 static tKFWUM_Status KfwumGetFileSize(unsigned char * pFileName,
                                                      unsigned long * pFileSize)
 {
    tKFWUM_Status status = KFWUM_STATUS_ERROR;
    FILE * pFileHandle;
 
-   pFileHandle = fopen(pFileName, "rb");
+   pFileHandle = fopen((const char *)pFileName, "rb");
 
    if(pFileHandle)
    {
@@ -468,6 +494,13 @@ static tKFWUM_Status KfwumGetFileSize(unsigned char * pFileName,
    return(status);
 }
 
+/* KfwumSetupBuffersFromFile  -  small buffers are used to store the file data
+ *
+ * @pFileName : filename ptr
+ * unsigned long : filesize
+ *
+ * returns KFWUM_STATUS_OK or KFWUM_STATUS_ERROR
+ */
 #define MAX_BUFFER_SIZE          1024*16
 static tKFWUM_Status KfwumSetupBuffersFromFile(unsigned char * pFileName,
                                                         unsigned long fileSize)
@@ -475,7 +508,7 @@ static tKFWUM_Status KfwumSetupBuffersFromFile(unsigned char * pFileName,
    tKFWUM_Status status = KFWUM_STATUS_OK;
    FILE * pFileHandle;
 
-   pFileHandle = fopen(pFileName, "rb");
+   pFileHandle = fopen((const char *)pFileName, "rb");
 
    if(pFileHandle)
    {
@@ -487,7 +520,7 @@ static tKFWUM_Status KfwumSetupBuffersFromFile(unsigned char * pFileName,
 
       for(qty=0;qty<count;qty++)
       {
-         KfwumShowProgress( "Reading Firmware from File", qty, count );
+         KfwumShowProgress((const unsigned char *)"Reading Firmware from File", qty, count );
          if(fread(&firmBuf[qty*MAX_BUFFER_SIZE], 1, MAX_BUFFER_SIZE ,pFileHandle)
             ==  MAX_BUFFER_SIZE)
          {
@@ -503,12 +536,20 @@ static tKFWUM_Status KfwumSetupBuffersFromFile(unsigned char * pFileName,
       }
       if(status == KFWUM_STATUS_OK)
       {
-         KfwumShowProgress( "Reading Firmware from File", 100, 100);
+         KfwumShowProgress((const unsigned char *)"Reading Firmware from File", 100, 100);
       }
    }
    return(status);
 }
 
+/* KfwumShowProgress  -  helper routine to display progress bar
+ *
+ * Converts current/total in percent
+ * 
+ * *task  : string identifying current operation
+ * current: progress
+ * total  : limit 
+ */
 #define PROG_LENGTH 42
 void KfwumShowProgress( const unsigned char * task,  unsigned long current ,
                                                            unsigned long total)
@@ -541,7 +582,7 @@ void KfwumShowProgress( const unsigned char * task,  unsigned long current ,
       printf("%s", spaces );
 
 
-      printf(" %3d %%\r",progress); /* total 7 bytes */
+      printf(" %3ld %%\r",progress); /* total 7 bytes */
 
       if( progress == 100 )
       {
@@ -551,7 +592,11 @@ void KfwumShowProgress( const unsigned char * task,  unsigned long current ,
    }
 }
 
-
+/* KfwumCalculateChecksumPadding
+ *
+ * TBD
+ * 
+ */
 static unsigned short KfwumCalculateChecksumPadding(unsigned char * pBuffer,
                                                        unsigned long totalSize)
 {
@@ -574,12 +619,24 @@ static unsigned short KfwumCalculateChecksumPadding(unsigned char * pBuffer,
 struct KfwumGetInfoResp {
    unsigned char protocolRevision;
    unsigned char controllerDeviceId;
-   unsigned char mode;
+   struct 
+   {
+      unsigned char mode:1;
+      unsigned char seqAdd:1;
+      unsigned char res : 6;
+   } byte;
    unsigned char firmRev1;
    unsigned char firmRev2;
    unsigned char numBank;
 } __attribute__ ((packed));
 
+
+/* KfwumGetInfo  -  Get Firmware Update Manager (FWUM) information
+ * 
+ * * intf  : IPMI interface 
+ * output  : when set to non zero, queried information is displayed
+ * pNumBank: output ptr for number of banks
+ */
 static tKFWUM_Status KfwumGetInfo(struct ipmi_intf * intf, unsigned char output,
                                                        unsigned char *pNumBank)
 {
@@ -620,7 +677,7 @@ static tKFWUM_Status KfwumGetInfo(struct ipmi_intf * intf, unsigned char output,
          printf("Firmware Revision         : %u.%u%u",
                                  pGetInfo->firmRev1, pGetInfo->firmRev2 >> 4,
                                                      pGetInfo->firmRev2 & 0x0f);
-         if(pGetInfo->mode != 0)
+         if(pGetInfo->byte.mode != 0)
          {
             printf(" - DEBUG BUILD\n");
          }
@@ -633,12 +690,19 @@ static tKFWUM_Status KfwumGetInfo(struct ipmi_intf * intf, unsigned char output,
       * pNumBank = pGetInfo->numBank;
       
       /* Determine wich type of download to use: */
-      /* Old FWUM or Old IPMC fw (data_len < 7) --> Address with small buffer size */
+      /* Old FWUM or Old IPMC fw (data_len < 7) -->
+          Address with small buffer size */
       if ( (pGetInfo->protocolRevision) <= 0x05 || (rsp->data_len < 7 ) )
       {
       	saveFirmwareInfo.downloadType = KFWUM_DOWNLOAD_TYPE_ADDRESS;
          saveFirmwareInfo.bufferSize   = KFWUM_SMALL_BUFFER;
          saveFirmwareInfo.overheadSize = KFWUM_OLD_CMD_OVERHEAD;
+
+         if(verbose)
+         {
+            printf("Protocol Revision          :");
+            printf(" <= 5 detected, adjusting buffers\n");
+         }
       }
       else /* Both fw are using the new protocol */
       {
@@ -646,20 +710,60 @@ static tKFWUM_Status KfwumGetInfo(struct ipmi_intf * intf, unsigned char output,
          saveFirmwareInfo.overheadSize = KFWUM_NEW_CMD_OVERHEAD;
          /* Buffer size depending on access type (Local or remote) */
          /* Look if we run remote or locally */
-         if ( intf->target_addr != IPMI_BMC_SLAVE_ADDR )
+
+         if(verbose)
          {
-            saveFirmwareInfo.bufferSize = KFWUM_SMALL_BUFFER;     	
+            printf("Protocol Revision          :");
+            printf(" > 5 optimizing buffers\n");
+         }
+
+         if(strstr(intf->name,"lan")!= NULL) /* also covers lanplus */
+         {
+            saveFirmwareInfo.bufferSize = KFWUM_SMALL_BUFFER;
+            if(verbose)
+            {
+               printf("IOL payload size           : %d\r\n"  ,
+                                                  saveFirmwareInfo.bufferSize);
+            }
+         }
+         else if
+         (
+           (strstr(intf->name,"open")!= NULL)
+           &&
+           intf->target_addr != IPMI_BMC_SLAVE_ADDR 
+           &&
+           (
+              intf->target_addr !=  intf->my_addr
+           )
+         )
+         {
+            saveFirmwareInfo.bufferSize = KFWUM_SMALL_BUFFER;
+            if(verbose)
+            {
+               printf("IPMB payload size          : %d\r\n"  , 
+                                                   saveFirmwareInfo.bufferSize);
+            }
          }
          else
          {
-         	saveFirmwareInfo.bufferSize = KFWUM_BIG_BUFFER;     	
+         	saveFirmwareInfo.bufferSize = KFWUM_BIG_BUFFER;
+            if(verbose)
+            {
+               printf("SMI payload size           : %d\r\n", 
+                                                  saveFirmwareInfo.bufferSize);
+            }
          }
       }
    }
    return status;
 }
 
-
+/* KfwumGetDeviceInfo  -  Get IPMC/Board information
+ * 
+ * * intf  : IPMI interface 
+ * output  : when set to non zero, queried information is displayed
+ * tKFWUM_BoardInfo: output ptr for IPMC/Board information
+ */
 static tKFWUM_Status KfwumGetDeviceInfo(struct ipmi_intf * intf,
                             unsigned char output, tKFWUM_BoardInfo * pBoardInfo)
 {
@@ -689,24 +793,8 @@ static tKFWUM_Status KfwumGetDeviceInfo(struct ipmi_intf * intf,
       }
    }
 
-#if 0
-   struct ipm_devid_rsp {
-	unsigned char device_id;
-	unsigned char device_revision;
-	unsigned char fw_rev1;
-	unsigned char fw_rev2;
-	unsigned char ipmi_version;
-	unsigned char adtl_device_support;
-	unsigned char manufacturer_id[3];
-	unsigned char product_id[2];
-	unsigned char aux_fw_rev[4];
-} __attribute__ ((packed));
-#endif
-
    if(status == KFWUM_STATUS_OK)
    {
-      unsigned long manufId;
-      unsigned short boardId;
       pGetDevId = (struct ipm_devid_rsp *) rsp->data;
       pBoardInfo->iana = IPM_DEV_MANUFACTURER_ID(pGetDevId->manufacturer_id);
       pBoardInfo->boardId = buf2short(pGetDevId->product_id);
@@ -739,11 +827,6 @@ static tKFWUM_Status KfwumGetDeviceInfo(struct ipmi_intf * intf,
    return status;
 }
 
-
-
-
-
-
 struct KfwumGetStatusResp {
    unsigned char bankState;
    unsigned char firmLengthLSB;
@@ -762,7 +845,10 @@ const struct valstr bankStateValS[] = {
    { 0x04, "Previous Good" }
 };
 
-
+/* KfwumGetStatus  -  Get (and prints) FWUM  banks information
+ * 
+ * * intf  : IPMI interface 
+ */
 static tKFWUM_Status KfwumGetStatus(struct ipmi_intf * intf)
 {
    tKFWUM_Status status = KFWUM_STATUS_OK;
@@ -821,7 +907,7 @@ static tKFWUM_Status KfwumGetStatus(struct ipmi_intf * intf)
             firmLength  = firmLength << 8;
             firmLength |= pGetStatus->firmLengthLSB;
 
-            printf("Firmware Length            : %d bytes\n", firmLength);
+            printf("Firmware Length            : %ld bytes\n", firmLength);
             printf("Firmware Revision          : %u.%u%u SDR %u\n",
                               pGetStatus->firmRev1,  pGetStatus->firmRev2 >> 4,
                            pGetStatus->firmRev2 & 0x0f, pGetStatus->firmRev3);
@@ -836,6 +922,10 @@ struct KfwumManualRollbackReq{
 } __attribute__ ((packed));
 
 
+/* KfwumManualRollback  -  Ask IPMC to rollback to previous version
+ * 
+ * * intf  : IPMI interface 
+ */
 static tKFWUM_Status KfwumManualRollback(struct ipmi_intf * intf)
 {
   tKFWUM_Status status = KFWUM_STATUS_OK;
@@ -952,22 +1042,26 @@ struct KfwumSaveFirmwareAddressReq
 struct KfwumSaveFirmwareSequenceReq
 {
  	unsigned char sequenceNumber;
-   unsigned char txBuf[KFWUM_BIG_BUFFER-KFWUM_NEW_CMD_OVERHEAD];
+   unsigned char txBuf[KFWUM_BIG_BUFFER];
 }__attribute__ ((packed));
 
+
+#define FWUM_SAVE_FIRMWARE_NO_RESPONSE_LIMIT ((unsigned char)6)
+
 static tKFWUM_Status KfwumSaveFirmwareImage(struct ipmi_intf * intf,
      unsigned char sequenceNumber, unsigned long address, unsigned char *pFirmBuf, 
-     unsigned char inBufLength)
+     unsigned char * pInBufLength)
 {
    tKFWUM_Status status = KFWUM_STATUS_OK;
    struct ipmi_rs * rsp;
    struct ipmi_rq req;
    unsigned char out = 0;
    unsigned char retry = 0;
-   unsigned char counter;
+   unsigned char noResponse = 0 ;
+   
    struct KfwumSaveFirmwareAddressReq  addressReq;
    struct KfwumSaveFirmwareSequenceReq sequenceReq;
-   
+
    do
    {
    	memset(&req, 0, sizeof(req));
@@ -979,26 +1073,46 @@ static tKFWUM_Status KfwumSaveFirmwareImage(struct ipmi_intf * intf,
       	addressReq.addressLSB  = address         & 0x000000ff;
          addressReq.addressMid  = (address >>  8) & 0x000000ff;
          addressReq.addressMSB  = (address >> 16) & 0x000000ff;
-         addressReq.numBytes    = inBufLength;
-         memcpy(addressReq.txBuf, pFirmBuf, inBufLength);
+         addressReq.numBytes    = (* pInBufLength);
+         memcpy(addressReq.txBuf, pFirmBuf, (* pInBufLength));
          req.msg.data = (unsigned char *) &addressReq;
-         req.msg.data_len = inBufLength+4;
+         req.msg.data_len = (* pInBufLength)+4;
    	}
       else
       {
       	sequenceReq.sequenceNumber = sequenceNumber;
-         memcpy(sequenceReq.txBuf, pFirmBuf, inBufLength);
+         memcpy(sequenceReq.txBuf, pFirmBuf, (* pInBufLength));
          req.msg.data = (unsigned char *) &sequenceReq;
-         req.msg.data_len = inBufLength+sizeof(unsigned char); /* + 1 => sequenceNumber*/
+         req.msg.data_len = (* pInBufLength)+sizeof(unsigned char); /* + 1 => sequenceNumber*/
       }
-      							      
+      						
       rsp = intf->sendrecv(intf, &req);
 
       if (!rsp)
       {
          printf("Error in FWUM Firmware Save Firmware Image Download Command\n");
-         status = KFWUM_STATUS_ERROR;
+
          out = 0;
+         status = KFWUM_STATUS_OK;
+         
+         /* With IOL, we don't receive "C7" on errors, instead we receive
+            nothing */
+         if(strstr(intf->name,"lan")!= NULL)
+         {
+            noResponse++;
+
+            if(noResponse < FWUM_SAVE_FIRMWARE_NO_RESPONSE_LIMIT )
+            {
+               (* pInBufLength) -= 1;
+               out = 0;
+            }
+            else
+            {
+               printf("Error, too many commands without response\n");
+               (* pInBufLength) = 0 ;               
+               out = 1;
+            }
+         } /* For other interface keep trying */
       }
       else if (rsp->ccode)
       {
@@ -1007,6 +1121,19 @@ static tKFWUM_Status KfwumSaveFirmwareImage(struct ipmi_intf * intf,
             status = KFWUM_STATUS_OK;
             sleep(1);
          }
+         else if(
+                  (rsp->ccode == 0xc7)
+                  ||
+                  (
+                     (rsp->ccode == 0xC3) &&
+                     (sequenceNumber == 0)
+                  )
+                )
+         {
+            (* pInBufLength) -= 1;
+            status = KFWUM_STATUS_OK;
+            retry = 1;
+         }
          else if(rsp->ccode == 0x82)
          {
             /* Double sent, continue */
@@ -1030,7 +1157,21 @@ static tKFWUM_Status KfwumSaveFirmwareImage(struct ipmi_intf * intf,
          {
             retry = 1;
             status = KFWUM_STATUS_OK;
-         }else
+         }
+         else if(rsp->ccode == 0xC3)
+         {
+            if(retry == 0)
+           {
+               retry = 1;
+               status = KFWUM_STATUS_OK;
+            }
+            else
+            {
+               status = KFWUM_STATUS_ERROR;
+               out = 1;
+            }
+         }
+         else
          {
             printf("FWUM Firmware Save Firmware Image Download returned %x\n",
                                                                      rsp->ccode);
@@ -1100,15 +1241,14 @@ static tKFWUM_Status KfwumUploadFirmware(struct ipmi_intf * intf,
    tKFWUM_Status status = KFWUM_STATUS_ERROR;
    unsigned long address    = 0x0;
    unsigned char writeSize;
+   unsigned char oldWriteSize;
    unsigned long lastAddress = 0;
    unsigned char sequenceNumber = 0;
    unsigned char retry = FWUM_MAX_UPLOAD_RETRY;
+   unsigned char isLengthValid = 1;
 
    do
    {
-      unsigned char  bytes;
-      unsigned char  chksum = 0;
-      
       writeSize = saveFirmwareInfo.bufferSize - saveFirmwareInfo.overheadSize;
            
       /* Reach the end */
@@ -1122,16 +1262,27 @@ static tKFWUM_Status KfwumUploadFirmware(struct ipmi_intf * intf,
          writeSize = (KFWUM_PAGE_SIZE - (address % KFWUM_PAGE_SIZE));
       }
 
-      status = KfwumSaveFirmwareImage(intf, sequenceNumber, address, &pBuffer[address], 
-                                                                     writeSize);
-
+      oldWriteSize = writeSize;
+      status = KfwumSaveFirmwareImage(intf, sequenceNumber, address, 
+                                                &pBuffer[address], &writeSize);
+ 
       if((status != KFWUM_STATUS_OK) && (retry-- != 0))
       {
          address = lastAddress;
          status = KFWUM_STATUS_OK;
       }
+      else if( writeSize == 0 )
+      {
+         status = KFWUM_STATUS_ERROR;     
+      }
       else
       {
+         if(writeSize != oldWriteSize)
+         {
+            printf("Adjusting length to %d bytes \n", writeSize);
+            saveFirmwareInfo.bufferSize -= (oldWriteSize - writeSize);
+         }
+         
          retry = FWUM_MAX_UPLOAD_RETRY;
          lastAddress = address;
          address+= writeSize;
@@ -1141,7 +1292,8 @@ static tKFWUM_Status KfwumUploadFirmware(struct ipmi_intf * intf,
       {
          if((address % 1024) == 0)
          {
-            KfwumShowProgress("Writting Firmware in Flash",address,totalSize);
+            KfwumShowProgress((const unsigned char *)\
+                                 "Writting Firmware in Flash",address,totalSize);
          }
          sequenceNumber++;
       }
@@ -1150,10 +1302,10 @@ static tKFWUM_Status KfwumUploadFirmware(struct ipmi_intf * intf,
 
    if(status == KFWUM_STATUS_OK)
    {
-      KfwumShowProgress( "Writting Firmware in Flash", 100 , 100 );
+      KfwumShowProgress((const unsigned char *)\
+                                       "Writting Firmware in Flash", 100 , 100 );
    }
 
-
    return(status);
 }
 
@@ -1461,32 +1613,12 @@ tKFWUM_Status KfwumValidFirmwareForBoard(tKFWUM_BoardInfo boardInfo,
 }
 
 
-tKFWUM_Status KfwumOutputInfo(tKFWUM_BoardInfo boardInfo,
+static void KfwumOutputInfo(tKFWUM_BoardInfo boardInfo,
                                                 tKFWUM_InFirmwareInfo firmInfo)
 {
-#if 0
-   unsigned long   fileSize;
-   unsigned short  checksum;
-   unsigned short  sumToRemoveFromChecksum;
-                                  /* Since the checksum is added in the bin
-                                  after the checksum is calculated, we
-                                  need to remove the each byte value.  This
-                                  byte will contain the addition of both bytes*/
-   tKFWUM_BoardList boardId;
-   unsigned char   deviceId;
-   unsigned char   tableVers;
-   unsigned char   implRev;
-   unsigned char   versMajor;
-   unsigned char   versMinor;
-   unsigned char   versSubMinor;
-   unsigned char   sdrRev;
-   tKFWUM_IanaList iana;
-#endif
-
    printf("Target Board Id            : %u\n",boardInfo.boardId);
    printf("Target IANA number         : %u\n",boardInfo.iana);
-   printf("File Size                  : %u bytes\n",firmInfo.fileSize);
+   printf("File Size                  : %lu bytes\n",firmInfo.fileSize);
    printf("Firmware Version           : %d.%d%d SDR %d\n",firmInfo.versMajor,
                    firmInfo.versMinor, firmInfo.versSubMinor, firmInfo.sdrRev);
-
 }
