@@ -71,9 +71,9 @@
 #endif
 
 #ifdef ENABLE_ALL_OPTIONS
-# define OPTION_STRING	"I:hVvcgsEao:H:P:f:U:p:C:L:A:t:m:S:l:b:e:k:O:"
+# define OPTION_STRING	"I:hVvcgsEao:H:d:P:f:U:p:C:L:A:t:m:S:l:b:e:k:O:"
 #else
-# define OPTION_STRING	"I:hVvcH:f:U:p:S:"
+# define OPTION_STRING	"I:hVvcH:f:U:p:d:S:"
 #endif
 
 extern int verbose;
@@ -216,6 +216,7 @@ ipmi_option_usage(const char * progname, struct ipmi_cmd * cmdlist, struct ipmi_
 	lprintf(LOG_NOTICE, "       -V             Show version information");
 	lprintf(LOG_NOTICE, "       -v             Verbose (can use multiple times)");
 	lprintf(LOG_NOTICE, "       -c             Display output in comma separated format");
+	lprintf(LOG_NOTICE, "       -d N           Specify a /dev/ipmiN device to use (default=0)");
 	lprintf(LOG_NOTICE, "       -I intf        Interface to use");
 	lprintf(LOG_NOTICE, "       -H hostname    Remote host name for LAN interface");
 	lprintf(LOG_NOTICE, "       -p port        Remote RMCP port [default=623]");
@@ -228,6 +229,7 @@ ipmi_option_usage(const char * progname, struct ipmi_cmd * cmdlist, struct ipmi_
 	lprintf(LOG_NOTICE, "       -C ciphersuite Cipher suite to be used by lanplus interface");
 	lprintf(LOG_NOTICE, "       -k key         Use Kg key for IPMIv2 authentication");
 	lprintf(LOG_NOTICE, "       -L level       Remote session privilege level [default=ADMINISTRATOR]");
+	lprintf(LOG_NOTICE, "                      Append a '+' to use name/privilege lookup in RAKP1");
 	lprintf(LOG_NOTICE, "       -A authtype    Force use of auth type NONE, PASSWORD, MD2, MD5 or OEM");
 	lprintf(LOG_NOTICE, "       -P password    Remote session password");
 	lprintf(LOG_NOTICE, "       -E             Read password from IPMI_PASSWORD environment variable");
@@ -269,6 +271,7 @@ ipmi_main(int argc, char ** argv,
 	uint8_t target_channel = 0;
 	uint8_t target_lun     = 0;
 	uint8_t my_addr = 0;
+	uint8_t lookupbit = 0x10;	/* use name-only lookup by default */
 	int authtype = -1;
 	char * tmp = NULL;
 	char * hostname = NULL;
@@ -281,6 +284,7 @@ ipmi_main(int argc, char ** argv,
 	char * kgkey    = NULL;
 	char * seloem   = NULL;
 	int port = 0;
+	int devnum = 0;
 	int cipher_suite_id = 3; /* See table 22-19 of the IPMIv2 spec */
 	int argflag, i, found;
 	int rc = -1;
@@ -323,6 +327,9 @@ ipmi_main(int argc, char ** argv,
 			rc = 0;
 			goto out_free;
 			break;
+		case 'd':
+			devnum = atoi(optarg);
+			break;
 		case 'p':
 			port = atoi(optarg);
 			break;
@@ -447,9 +454,15 @@ ipmi_main(int argc, char ** argv,
 			}
 			break;
 		case 'L':
+			i = strlen(optarg);
+			if ((i > 0) && (optarg[i-1] == '+')) {
+				lookupbit = 0;
+				optarg[i-1] = 0;
+			}
 			privlvl = str2val(optarg, ipmi_privlvl_vals);
-			if (privlvl == 0xFF)
+			if (privlvl == 0xFF) {
 				lprintf(LOG_WARN, "Invalid privilege level %s", optarg);
+			}
 			break;
 		case 'A':
 			authtype = str2val(optarg, ipmi_authtype_session_vals);
@@ -564,6 +577,7 @@ ipmi_main(int argc, char ** argv,
 		ipmi_intf_session_set_privlvl(intf,
 		      IPMI_SESSION_PRIV_ADMIN);	/* default */
 
+	ipmi_intf_session_set_lookupbit(intf, lookupbit);
 	ipmi_intf_session_set_sol_escape_char(intf, sol_escape_char);
 	ipmi_intf_session_set_cipher_suite_id(intf, cipher_suite_id);
 
@@ -573,6 +587,8 @@ ipmi_main(int argc, char ** argv,
 	/* setup destination channel if given */
 	intf->target_channel = target_channel ;
 
+	intf->devnum = devnum;
+
 	/* setup IPMB local and target address if given */
 	intf->my_addr = my_addr ? : IPMI_BMC_SLAVE_ADDR;
 	if (target_addr > 0) {
