@@ -730,6 +730,7 @@ ipmi_pef_get_status(struct ipmi_intf * intf)
 	struct ipmi_rq req;
 	struct pef_cfgparm_selector psel;
 	char tbuf[40];
+	uint32_t timei;
 	time_t ts;
 
 	memset(&req, 0, sizeof(req));
@@ -741,11 +742,12 @@ ipmi_pef_get_status(struct ipmi_intf * intf)
 			"Last S/W processed ID");
 		return;
 	}
+	memcpy(&timei, rsp->data, sizeof(timei));
 #if WORDS_BIGENDIAN
-	ts = (time_t)(rsp->data[0]<<24 + rsp->data[1]<<16 + rsp->data[2]<<8 + rsp->data[3]);
-#else
-	ts = (time_t)(*(long *)rsp->data);
+	timei = BSWAP_32(timei);
 #endif
+	ts = (time_t)timei;
+
 	strftime(tbuf, sizeof(tbuf), "%m/%d/%Y %H:%M:%S", localtime(&ts));
 
 	ipmi_pef_print_str("Last SEL addition", tbuf);
@@ -843,6 +845,7 @@ int ipmi_pef_main(struct ipmi_intf * intf, int argc, char ** argv)
 	// PEF subcommand handling
 	*/
 	int help = 0;
+    int rc = 0;
 
 	if (!argc || !strncmp(argv[0], "info", 4))
 		ipmi_pef_get_info(intf);
@@ -856,11 +859,14 @@ int ipmi_pef_main(struct ipmi_intf * intf, int argc, char ** argv)
 		ipmi_pef_list_entries(intf);
 	else {
 		help = 1;
+        rc   = -1;
 		lprintf(LOG_ERR, "Invalid PEF command: '%s'\n", argv[0]);
 	}
+
 	if (help)
 		lprintf(LOG_NOTICE, "PEF commands: info status policy list");
 	else if (!verbose)
 		printf("\n");
-	return 0;
+
+	return rc;
 }
