@@ -43,6 +43,7 @@
 
 extern int verbose;
 
+#define SCANNING_DISABLED       0x40
 #define READING_UNAVAILABLE	0x20
 
 static
@@ -110,6 +111,8 @@ ipmi_sensor_print_full_discrete(struct ipmi_intf *intf,
 		return -1;
 	} else if (rsp->ccode > 0 || (rsp->data[1] & READING_UNAVAILABLE)) {
 		validread = 0;
+	} else if (!(rsp->data[1] & SCANNING_DISABLED)) {
+	  	validread = 0;
 	} else {
 		/* convert RAW reading into units */
 		val = rsp->data[0];
@@ -143,16 +146,19 @@ ipmi_sensor_print_full_discrete(struct ipmi_intf *intf,
 			printf(" Sensor Type (Discrete): %s\n",
 			       ipmi_sdr_get_sensor_type_desc(sensor->sensor.
 							     type));
-			ipmi_sdr_print_discrete_state("States Asserted",
-						      sensor->sensor.type,
-						      sensor->event_type,
-						      rsp->data[2],
-						      rsp->data[3]);
-			printf("\n");
+         if( validread )
+         {
+            ipmi_sdr_print_discrete_state("States Asserted",
+												      sensor->sensor.type,
+												      sensor->event_type,
+												      rsp->data[2],
+												      rsp->data[3]);
+				printf("\n");
+         }
 		}
 	}
 
-	return 0;
+	return (validread ? 0 : -1 );
 }
 
 static int
@@ -190,6 +196,8 @@ ipmi_sensor_print_full_analog(struct ipmi_intf *intf,
 		return -1;
 	} else if (rsp->ccode || (rsp->data[1] & READING_UNAVAILABLE)) {
 		validread = 0;
+	} else if (!(rsp->data[1] & SCANNING_DISABLED)) {
+		validread = 0;
 	} else {
 		/* convert RAW reading into units */
 		val = (rsp->data[0] > 0)
@@ -385,7 +393,7 @@ ipmi_sensor_print_full_analog(struct ipmi_intf *intf,
 		}
 	}
 
-	return 0;
+	return (validread ? 0 : -1 );
 }
 
 int
@@ -427,6 +435,8 @@ ipmi_sensor_print_compact(struct ipmi_intf *intf,
 		return -1;
 	} else if (rsp->ccode || (rsp->data[1] & READING_UNAVAILABLE)) {
 		validread = 0;
+	} else if (!(rsp->data[1] & SCANNING_DISABLED)) {
+		validread = 0;
 	} else {
 		/* convert RAW reading into units */
 		val = rsp->data[0];
@@ -461,16 +471,20 @@ ipmi_sensor_print_compact(struct ipmi_intf *intf,
 			printf(" Sensor Type (Discrete): %s\n",
 			       ipmi_sdr_get_sensor_type_desc(sensor->sensor.
 							     type));
-			ipmi_sdr_print_discrete_state("States Asserted",
-						      sensor->sensor.type,
-						      sensor->event_type,
-						      rsp->data[2],
-						      rsp->data[3]);
-			printf("\n");
+
+         if(validread)
+         {
+            ipmi_sdr_print_discrete_state("States Asserted",
+						      						sensor->sensor.type,
+												      sensor->event_type,
+												      rsp->data[2],
+												      rsp->data[3]);
+            printf("\n");
+         }
 		}
 	}
 
-	return 0;
+	return (validread ? 0 : -1 );
 }
 
 static int
@@ -771,6 +785,8 @@ ipmi_sensor_get_reading(struct ipmi_intf *intf, int argc, char **argv)
 				continue;
 			} else if (rsp->data[1] & READING_UNAVAILABLE) {
 				continue;
+			} else if (!(rsp->data[1] & SCANNING_DISABLED)) {
+				continue;
 			} else if (rsp->data[0] > 0) {
 				/* convert RAW reading into units */
 				val = sdr_convert_sensor_reading(sdr->record.full, rsp->data[0]);
