@@ -1,4 +1,4 @@
-/*
+/*                                 
  * Copyright (c) 2003 Sun Microsystems, Inc.  All Rights Reserved.
  * 
  * Redistribution and use in source and binary forms, with or without
@@ -88,10 +88,58 @@ const struct valstr sol_parameter_vals[] = {
 static struct timeval _start_keepalive;
 static struct termios _saved_tio;
 static int            _in_raw_mode = 0;
+static int            _disable_keepalive = 0;
+static int            _use_sol_for_keepalive = 0;
 
 extern int verbose;
 
-static uint8_t G_u8ActiveSOL = 0;
+/*
+ * ipmi_sol_payload_access
+ */
+int
+ipmi_sol_payload_access(struct ipmi_intf * intf,
+			uint8_t channel,
+			uint8_t userid,
+			int enable)
+{
+	struct ipmi_rq req;
+	struct ipmi_rs *rsp;
+	uint8_t data[6];
+
+	memset(&req, 0, sizeof(req));
+	req.msg.netfn	 = IPMI_NETFN_APP;
+	req.msg.cmd	 = IPMI_SET_USER_PAYLOAD_ACCESS;
+	req.msg.data	 = data;
+	req.msg.data_len = 6;
+
+	memset(data, 0, 6);
+
+	data[0] = channel & 0xf;	/* channel */
+	data[1] = userid & 0x3f;	/* user id */
+	if (!enable)
+		data[1] |= 0x40;	/* disable */
+	data[2] = 0x02;			/* payload 1 is SOL */
+
+	rsp = intf->sendrecv(intf, &req);
+
+	if (NULL != rsp) {
+		switch (rsp->ccode) {
+			case 0x00:
+				return 0;
+			default:
+				lprintf(LOG_ERR, "Error %sabling SOL payload for user %d on channel %d: %s",
+					enable ? "en" : "dis", userid, channel,
+					val2str(rsp->ccode, completion_code_vals));
+				break;
+		}
+	} else {
+		lprintf(LOG_ERR, "Error %sabling SOL payload for user %d on channel %d",
+			enable ? "en" : "dis", userid, channel);
+	}
+
+	return -1;
+}
+
 
 /*
  * ipmi_get_sol_info
@@ -104,9 +152,9 @@ ipmi_get_sol_info(
 {
 	struct ipmi_rs * rsp;
 	struct ipmi_rq req;
-	uint8_t data[4];	
+	uint8_t data[4];
 
-	memset(&req, 0, sizeof(req)); 
+	memset(&req, 0, sizeof(req));
 	req.msg.netfn    = IPMI_NETFN_TRANSPORT;
 	req.msg.cmd      = IMPI_GET_SOL_CONFIG_PARAMETERS;
 	req.msg.data_len = 4;
@@ -121,12 +169,12 @@ ipmi_get_sol_info(
 	data[1] = SOL_PARAMETER_SET_IN_PROGRESS; /* parameter selector */
 	data[2] = 0x00;                          /* set selector       */
 	data[3] = 0x00;                          /* block selector     */
-	
+
 	rsp = intf->sendrecv(intf, &req);
 
 	if (NULL != rsp) {
 		switch (rsp->ccode) {
-			case 0x00: 
+			case 0x00:
 				if (rsp->data_len == 2) {
 					params->set_in_progress = rsp->data[1];
 				} else {
@@ -137,12 +185,12 @@ ipmi_get_sol_info(
 				}
 				break;
 			case 0x80:
-				lprintf(LOG_ERR, "Info: SOL parameter '%s' not supported", 
+				lprintf(LOG_ERR, "Info: SOL parameter '%s' not supported",
 				    val2str(data[1], sol_parameter_vals));
 				break;
 			default:
 				lprintf(LOG_ERR, "Error requesting SOL parameter '%s': %s",
-					val2str(data[1], sol_parameter_vals),	
+					val2str(data[1], sol_parameter_vals),
 					val2str(rsp->ccode, completion_code_vals));
 				return -1;
 		}
@@ -159,12 +207,12 @@ ipmi_get_sol_info(
 	data[1] = SOL_PARAMETER_SOL_ENABLE; /* parameter selector */
 	data[2] = 0x00;                     /* set selector       */
 	data[3] = 0x00;                     /* block selector     */
-	
+
 	rsp = intf->sendrecv(intf, &req);
 
 	if (NULL != rsp) {
 		switch (rsp->ccode) {
-			case 0x00: 
+			case 0x00:
 				if (rsp->data_len == 2) {
 					params->enabled = rsp->data[1];
 				} else {
@@ -175,7 +223,7 @@ ipmi_get_sol_info(
 				}
 				break;
 			case 0x80:
-				lprintf(LOG_ERR, "Info: SOL parameter '%s' not supported", 
+				lprintf(LOG_ERR, "Info: SOL parameter '%s' not supported",
 				    val2str(data[1], sol_parameter_vals));
 				break;
 			default:
@@ -197,11 +245,11 @@ ipmi_get_sol_info(
 	data[1] = SOL_PARAMETER_SOL_AUTHENTICATION; /* parameter selector */
 	data[2] = 0x00;                             /* set selector       */
 	data[3] = 0x00;                             /* block selector     */
-	
+
 	rsp = intf->sendrecv(intf, &req);
 	if (NULL != rsp) {
 		switch (rsp->ccode) {
-			case 0x00: 
+			case 0x00:
 				if (rsp->data_len == 2) {
 					params->force_encryption     = ((rsp->data[1] & 0x80)? 1 : 0);
 					params->force_authentication = ((rsp->data[1] & 0x40)? 1 : 0);
@@ -214,7 +262,7 @@ ipmi_get_sol_info(
 				}
 				break;
 			case 0x80:
-				lprintf(LOG_ERR, "Info: SOL parameter '%s' not supported", 
+				lprintf(LOG_ERR, "Info: SOL parameter '%s' not supported",
 				    val2str(data[1], sol_parameter_vals));
 				break;
 			default:
@@ -236,11 +284,11 @@ ipmi_get_sol_info(
 	data[1] = SOL_PARAMETER_CHARACTER_INTERVAL; /* parameter selector */
 	data[2] = 0x00;                             /* set selector       */
 	data[3] = 0x00;                             /* block selector     */
-	
+
 	rsp = intf->sendrecv(intf, &req);
 	if (NULL != rsp) {
 		switch (rsp->ccode) {
-			case 0x00: 
+			case 0x00:
 				if (rsp->data_len == 3) {
 					params->character_accumulate_level = rsp->data[1];
 					params->character_send_threshold   = rsp->data[2];
@@ -252,7 +300,7 @@ ipmi_get_sol_info(
 				}
 				break;
 			case 0x80:
-				lprintf(LOG_ERR, "Info: SOL parameter '%s' not supported", 
+				lprintf(LOG_ERR, "Info: SOL parameter '%s' not supported",
 				    val2str(data[1], sol_parameter_vals));
 				break;
 			default:
@@ -274,14 +322,14 @@ ipmi_get_sol_info(
 	data[1] = SOL_PARAMETER_SOL_RETRY; /* parameter selector */
 	data[2] = 0x00;                    /* set selector       */
 	data[3] = 0x00;                    /* block selector     */
-	
+
 	rsp = intf->sendrecv(intf, &req);
 	if (NULL != rsp) {
 		switch (rsp->ccode) {
-			case 0x00: 
+			case 0x00:
 				if (rsp->data_len == 3) {
 					params->retry_count    = rsp->data[1];
-					params->retry_interval = rsp->data[2]; 
+					params->retry_interval = rsp->data[2];
 				} else {
 					lprintf(LOG_ERR, "Error: Unexpected data length (%d) received "
 						   "for SOL parameter '%s'",
@@ -290,7 +338,7 @@ ipmi_get_sol_info(
 				}
 				break;
 			case 0x80:
-				lprintf(LOG_ERR, "Info: SOL parameter '%s' not supported", 
+				lprintf(LOG_ERR, "Info: SOL parameter '%s' not supported",
 				    val2str(data[1], sol_parameter_vals));
 				break;
 			default:
@@ -312,11 +360,11 @@ ipmi_get_sol_info(
  	data[1] = SOL_PARAMETER_SOL_NON_VOLATILE_BIT_RATE; /* parameter selector */
  	data[2] = 0x00;                                    /* set selector       */
 	data[3] = 0x00;                                    /* block selector     */
-	
+
 	rsp = intf->sendrecv(intf, &req);
 	if (NULL != rsp) {
 		switch (rsp->ccode) {
-			case 0x00: 
+			case 0x00:
 				if (rsp->data_len == 2) {
 					params->non_volatile_bit_rate = rsp->data[1] & 0x0F;
 				} else {
@@ -327,7 +375,7 @@ ipmi_get_sol_info(
 				}
 				break;
 			case 0x80:
-				lprintf(LOG_ERR, "Info: SOL parameter '%s' not supported", 
+				lprintf(LOG_ERR, "Info: SOL parameter '%s' not supported",
 				    val2str(data[1], sol_parameter_vals));
 				break;
 			default:
@@ -349,11 +397,11 @@ ipmi_get_sol_info(
 	data[1] = SOL_PARAMETER_SOL_VOLATILE_BIT_RATE; /* parameter selector */
 	data[2] = 0x00;                                /* set selector       */
 	data[3] = 0x00;                                /* block selector     */
-	
+
 	rsp = intf->sendrecv(intf, &req);
 	if (NULL != rsp) {
 		switch (rsp->ccode) {
-			case 0x00: 
+			case 0x00:
 				if (rsp->data_len == 2) {
 					params->volatile_bit_rate = rsp->data[1] & 0x0F;
 				} else {
@@ -364,7 +412,7 @@ ipmi_get_sol_info(
 				}
 				break;
 			case 0x80:
-				lprintf(LOG_ERR, "Info: SOL parameter '%s' not supported", 
+				lprintf(LOG_ERR, "Info: SOL parameter '%s' not supported",
 				    val2str(data[1], sol_parameter_vals));
 				break;
 			default:
@@ -386,11 +434,11 @@ ipmi_get_sol_info(
 	data[1] = SOL_PARAMETER_SOL_PAYLOAD_CHANNEL; /* parameter selector */
 	data[2] = 0x00;                              /* set selector       */
 	data[3] = 0x00;                              /* block selector     */
-	
+
 	rsp = intf->sendrecv(intf, &req);
 	if (NULL != rsp) {
 		switch (rsp->ccode) {
-			case 0x00: 
+			case 0x00:
 				if (rsp->data_len == 2) {
 					params->payload_channel = rsp->data[1];
 				} else {
@@ -401,7 +449,7 @@ ipmi_get_sol_info(
 				}
 				break;
 			case 0x80:
-				lprintf(LOG_ERR, "Info: SOL parameter '%s' not supported - defaulting to 0x%02x", 
+				lprintf(LOG_ERR, "Info: SOL parameter '%s' not supported - defaulting to 0x%02x",
 				    val2str(data[1], sol_parameter_vals), channel);
 				params->payload_channel = channel;
 				break;
@@ -424,11 +472,11 @@ ipmi_get_sol_info(
 	data[1] = SOL_PARAMETER_SOL_PAYLOAD_PORT; /* parameter selector */
 	data[2] = 0x00;                           /* set selector       */
 	data[3] = 0x00;                           /* block selector     */
-	
+
 	rsp = intf->sendrecv(intf, &req);
 	if (NULL != rsp) {
 		switch (rsp->ccode) {
-			case 0x00: 
+			case 0x00:
 				if (rsp->data_len == 3) {
 					params->payload_port = (rsp->data[1]) |	(rsp->data[2] << 8);
 				} else {
@@ -439,7 +487,7 @@ ipmi_get_sol_info(
 				}
 				break;
 			case 0x80:
-				lprintf(LOG_ERR, "Info: SOL parameter '%s' not supported - defaulting to %d", 
+				lprintf(LOG_ERR, "Info: SOL parameter '%s' not supported - defaulting to %d",
 				    val2str(data[1], sol_parameter_vals), intf->session->port);
 				params->payload_port = intf->session->port;
 				break;
@@ -468,7 +516,7 @@ ipmi_print_sol_info(struct ipmi_intf * intf, uint8_t channel)
 	struct sol_config_parameters params = {0};
 	if (ipmi_get_sol_info(intf, channel, &params))
 		return -1;
-	
+
 	if (csv_output)
 	{
 		printf("%s,",
@@ -543,22 +591,22 @@ ipmi_print_sol_info(struct ipmi_intf * intf, uint8_t channel)
  */
 static int
 ipmi_sol_set_param(struct ipmi_intf * intf,
-				   uint8_t      channel,
-				   const char       * param,
-				   const char       * value)
+		   uint8_t            channel,
+		   const char       * param,
+		   const char       * value,
+		   uint8_t            guarded)
 {
 	struct ipmi_rs * rsp;
 	struct ipmi_rq   req;
-	uint8_t    data[4];	 
-	int              bGuarded = 1; /* Use set-in-progress indicator? */
+	uint8_t          data[4];
+	int              bGuarded = guarded; /* Use set-in-progress indicator? */
 
-   memset(&req, 0, sizeof(req));
-	req.msg.netfn    = IPMI_NETFN_TRANSPORT;           /* 0x0c */ 
+	memset(&req, 0, sizeof(req));
+	req.msg.netfn    = IPMI_NETFN_TRANSPORT;           /* 0x0c */
 	req.msg.cmd      = IMPI_SET_SOL_CONFIG_PARAMETERS; /* 0x21 */
 	req.msg.data     = data;
-	
+
 	data[0] = channel;
-	
 
 	/*
 	 * set-in-progress
@@ -638,11 +686,11 @@ ipmi_sol_set_param(struct ipmi_intf * intf,
 				   param);
 			return -1;
 		}
-			
+
 		data[2] |= params.force_authentication? 0x40 : 0x00;
-		data[2] |= params.privilege_level; 
+		data[2] |= params.privilege_level;
 	}
-	
+
 
 	/*
 	 * force-payload-authentication
@@ -676,9 +724,9 @@ ipmi_sol_set_param(struct ipmi_intf * intf,
 		}
 
 		data[2] |= params.force_encryption? 0x80 : 0x00;
-		data[2] |= params.privilege_level; 
+		data[2] |= params.privilege_level;
 	}
-	
+
 
 	/*
 	 * privilege-level
@@ -738,7 +786,7 @@ ipmi_sol_set_param(struct ipmi_intf * intf,
 				   param);
 			return -1;
 		}
-		
+
 		data[3] = params.character_send_threshold;
 	}
 
@@ -764,7 +812,7 @@ ipmi_sol_set_param(struct ipmi_intf * intf,
 
 		data[2] = params.character_accumulate_level;
 	}
-	
+
 
 	/*
 	 * retry-count
@@ -807,11 +855,11 @@ ipmi_sol_set_param(struct ipmi_intf * intf,
 				   param);
 			return -1;
 		}
-		
+
 		data[2] = params.retry_count;
 	}
 
-	
+
 	/*
 	 * non-volatile-bit-rate
 	 */
@@ -853,7 +901,7 @@ ipmi_sol_set_param(struct ipmi_intf * intf,
 			return -1;
 		}
 	}
-	
+
 
 	/*
 	 * volatile-bit-rate
@@ -896,7 +944,6 @@ ipmi_sol_set_param(struct ipmi_intf * intf,
 			return -1;
 		}
 	}
-	
 	else
 	{
 		lprintf(LOG_ERR, "Error: invalid SOL parameter %s", param);
@@ -909,9 +956,10 @@ ipmi_sol_set_param(struct ipmi_intf * intf,
 	 */
 	if (bGuarded &&
 		(ipmi_sol_set_param(intf,
-							channel,
-							"set-in-progress",
-							"set-in-progress")))
+				    channel,
+				    "set-in-progress",
+				    "set-in-progress",
+				    bGuarded)))
 	{
 		lprintf(LOG_ERR, "Error: set of parameter \"%s\" failed", param);
 		return -1;
@@ -924,23 +972,45 @@ ipmi_sol_set_param(struct ipmi_intf * intf,
 	if (rsp == NULL) {
 		lprintf(LOG_ERR, "Error setting SOL parameter '%s'", param);
 		return -1;
-	}		
+	}
 
-	if (rsp->ccode > 0) {
-		lprintf(LOG_ERR, "Error setting SOL parameter '%s': %s",
-			   param, val2str(rsp->ccode, completion_code_vals));
+	if (!(!strncmp(param, "set-in-progress", 15) && !strncmp(value, "commit-write", 12)) &&
+	    rsp->ccode > 0) {
+		switch (rsp->ccode) {
+		case 0x80:
+			lprintf(LOG_ERR, "Error setting SOL parameter '%s': "
+				"Parameter not supported", param);
+			break;
+		case 0x81:
+			lprintf(LOG_ERR, "Error setting SOL parameter '%s': "
+				"Attempt to set set-in-progress when not in set-complete state",
+				param);
+			break;
+		case 0x82:
+			lprintf(LOG_ERR, "Error setting SOL parameter '%s': "
+				"Attempt to write read-only parameter", param);
+			break;
+		case 0x83:
+			lprintf(LOG_ERR, "Error setting SOL parameter '%s': "
+				"Attempt to read write-only parameter", param);
+			break;
+		default:
+			lprintf(LOG_ERR, "Error setting SOL parameter '%s' to '%s': %s",
+				param, value, val2str(rsp->ccode, completion_code_vals));
+			break;
+		}
 
 		if (bGuarded &&
 			(ipmi_sol_set_param(intf,
-								channel,
-								"set-in-progress",
-								"set-complete")))
+					    channel,
+					    "set-in-progress",
+					    "set-complete",
+					    bGuarded)))
 		{
 			lprintf(LOG_ERR, "Error could not set \"set-in-progress\" "
 				   "to \"set-complete\"");
-			return -1;
 		}
-		
+
 		return -1;
 	}
 
@@ -951,16 +1021,18 @@ ipmi_sol_set_param(struct ipmi_intf * intf,
 	 */
 	if (bGuarded)
 		ipmi_sol_set_param(intf,
-						   channel,
-						   "set-in-progress",
-						   "commit-write");
+				   channel,
+				   "set-in-progress",
+				   "commit-write",
+				   bGuarded);
 
 
 	if (bGuarded &&
  		ipmi_sol_set_param(intf,
-		 				   channel,
-			 			   "set-in-progress",
-						   "set-complete"))
+				   channel,
+				   "set-in-progress",
+				   "set-complete",
+				   bGuarded))
 	{
 		lprintf(LOG_ERR, "Error could not set \"set-in-progress\" "
 			   "to \"set-complete\"");
@@ -995,8 +1067,7 @@ enter_raw_mode(void)
 	}
 	_saved_tio = tio;
 	tio.c_iflag |= IGNPAR;
-	tio.c_iflag &= ~(ISTRIP | INLCR | IGNCR | ICRNL | IXON | IXANY | IXOFF)\
-		;
+	tio.c_iflag &= ~(ISTRIP | INLCR | IGNCR | ICRNL | IXON | IXANY | IXOFF);
 	tio.c_lflag &= ~(ISIG | ICANON | ECHO | ECHOE | ECHOK | ECHONL);
 	//	#ifdef IEXTEN
 	tio.c_lflag &= ~IEXTEN;
@@ -1084,9 +1155,21 @@ printSolEscapeSequences(struct ipmi_intf * intf)
 static void
 output(struct ipmi_rs * rsp)
 {
-	if (rsp)
+	/* Add checks to make sure it is actually SOL data, in general I see
+	 * outside code mostly trying to guard against this happening, but
+	 * some places fail to do so, so I do so here to make sure nothing gets
+	 * through.  If non-sol data comes through here, there is probably 
+	 * a packet that won't get processed somewhere else, but the alternative
+	 * of outputting corrupt data is worse.  Generally I see the get device
+	 * id response make it here somehow.  I assume it is a heartbeat and the
+	 * other code will retry if it cares about the response and misses it.
+	 */
+	if (rsp &&
+	    (rsp->session.authtype    == IPMI_SESSION_AUTHTYPE_RMCP_PLUS) &&
+	    (rsp->session.payloadtype == IPMI_PAYLOAD_TYPE_SOL))
 	{
 		int i;
+
 		for (i = 0; i < rsp->data_len; ++i)
 			putc(rsp->data[i], stdout);
 
@@ -1104,9 +1187,9 @@ ipmi_sol_deactivate(struct ipmi_intf * intf)
 {
 	struct ipmi_rs * rsp;
 	struct ipmi_rq   req;
-	uint8_t    data[6];	 
+	uint8_t          data[6];
 
-   memset(&req, 0, sizeof(req));
+	memset(&req, 0, sizeof(req));
 	req.msg.netfn    = IPMI_NETFN_APP;
 	req.msg.cmd      = IPMI_DEACTIVATE_PAYLOAD;
 	req.msg.data_len = 6;
@@ -1126,7 +1209,7 @@ ipmi_sol_deactivate(struct ipmi_intf * intf)
 
 	if (NULL != rsp) {
 		switch (rsp->ccode) {
-			case 0x00: 
+			case 0x00:
 				return 0;
 			case 0x80:
 				lprintf(LOG_ERR, "Info: SOL payload already de-activated");
@@ -1144,7 +1227,7 @@ ipmi_sol_deactivate(struct ipmi_intf * intf)
 	}
 
 	return -1;
-}	
+}
 
 
 
@@ -1184,7 +1267,7 @@ processSolUserInput(
 
 		if (escape_pending){
 			escape_pending = 0;
-			
+
 			/*
 			 * Process a possible escape sequence.
 			 */
@@ -1242,7 +1325,7 @@ processSolUserInput(
 		last_was_cr = (ch == '\r' || ch == '\n');
 	}
 
-	
+
 	/*
 	 * If there is anything left to process we dispatch it to the BMC,
 	 * send intf->session->sol_data.max_outbound_payload_size bytes
@@ -1254,8 +1337,9 @@ processSolUserInput(
 		int try = 0;
 
 		while (try < intf->session->retry) {
-		
+
 			v2_payload.payload.sol_packet.character_count = length;
+
 			rsp = intf->send_sol(intf, &v2_payload);
 
 			if (rsp)
@@ -1284,23 +1368,53 @@ processSolUserInput(
 	return retval;
 }
 
-
 static int
-ipmi_sol_keepalive(struct ipmi_intf * intf)
+ipmi_sol_keepalive_using_sol(struct ipmi_intf * intf)
 {
+	struct ipmi_v2_payload v2_payload;
+   struct ipmi_rs * rsp = NULL;
 	struct timeval end;
+
 	int ret = 0;
 
+	if (_disable_keepalive)
+		return 0;
+
 	gettimeofday(&end, 0);
 
 	if (end.tv_sec - _start_keepalive.tv_sec > SOL_KEEPALIVE_TIMEOUT) {
-		ret = intf->keepalive(intf);
-		gettimeofday(&_start_keepalive, 0);
-	}
+	   memset(&v2_payload, 0, sizeof(v2_payload));
 
+      v2_payload.payload.sol_packet.character_count = 0;
+
+      rsp = intf->send_sol(intf, &v2_payload);
+
+		gettimeofday(&_start_keepalive, 0);
+   }
 	return ret;
 }
 
+static int
+ipmi_sol_keepalive_using_getdeviceid(struct ipmi_intf * intf)
+{
+	struct timeval  end;
+	int ret = 0;
+
+	if (_disable_keepalive)
+		return 0;
+
+	gettimeofday(&end, 0);
+
+	if (end.tv_sec - _start_keepalive.tv_sec > SOL_KEEPALIVE_TIMEOUT) {
+	   ret = intf->keepalive(intf);
+		gettimeofday(&_start_keepalive, 0);
+   }
+	return ret;
+}
+
+
+
+
 
 /*
  * ipmi_sol_red_pill
@@ -1308,7 +1422,6 @@ ipmi_sol_keepalive(struct ipmi_intf * intf)
 static int
 ipmi_sol_red_pill(struct ipmi_intf * intf)
 {
-	
 	char   * buffer;
 	int    numRead;
 	int    bShouldExit       = 0;
@@ -1321,12 +1434,12 @@ ipmi_sol_red_pill(struct ipmi_intf * intf)
 
 	buffer = (char*)malloc(buffer_size);
 	if (buffer == NULL) {
-		lprintf(LOG_ERR, "ipmitool: malloc failure");
+		lprintf(LOG_ERR, "ipmitool: malloc failure"); 
 		return -1;
 	}
 
 	/* Initialize keepalive start time */
-	gettimeofday(&_start_keepalive, 0);	
+	gettimeofday(&_start_keepalive, 0);
 
 	enter_raw_mode();
 
@@ -1337,7 +1450,15 @@ ipmi_sol_red_pill(struct ipmi_intf * intf)
 		FD_SET(intf->fd, &read_fds);
 
 		/* Send periodic keepalive packet */
-		keepAliveRet = ipmi_sol_keepalive(intf);
+      if(_use_sol_for_keepalive == 0)
+      {
+         keepAliveRet = ipmi_sol_keepalive_using_getdeviceid(intf);
+      }
+      else
+      {
+         keepAliveRet = ipmi_sol_keepalive_using_sol(intf);
+      }
+		
 		if (keepAliveRet != 0)
 		{
 			/* no response to keepalive message */
@@ -1370,11 +1491,11 @@ ipmi_sol_red_pill(struct ipmi_intf * intf)
 				numRead = read(fileno(stdin),
 							   buffer,
 							   buffer_size);
-				
+
 				if (numRead > 0)
 				{
 					int rc = processSolUserInput(intf, (uint8_t *)buffer, numRead);
-					
+
 					if (rc)
 					{
 						if (rc < 0)
@@ -1401,10 +1522,12 @@ ipmi_sol_red_pill(struct ipmi_intf * intf)
 					bShouldExit = bBmcClosedSession = 1;
 				}
 				else
+				{
 					output(rs);
+				}
  			}
 
-			
+
 			/*
 			 * ERROR in select
 			 */
@@ -1414,7 +1537,7 @@ ipmi_sol_red_pill(struct ipmi_intf * intf)
 				bShouldExit = 1;
 			}
 		}
-	}		
+	}
 
 	leave_raw_mode();
 
@@ -1449,7 +1572,7 @@ ipmi_sol_activate(struct ipmi_intf * intf, int looptest, int interval)
 	struct ipmi_rs * rsp;
 	struct ipmi_rq   req;
 	struct activate_payload_rsp ap_rsp;
-	uint8_t    data[6];	 
+	uint8_t    data[6];
 	uint8_t    bSolEncryption     = 1;
 	uint8_t    bSolAuthentication = 1;
 
@@ -1497,7 +1620,6 @@ ipmi_sol_activate(struct ipmi_intf * intf, int looptest, int interval)
 	data[4] = 0x00; /* reserved */
 	data[5] = 0x00; /* reserved */
 
-	 G_u8ActiveSOL = 1; 
 	rsp = intf->sendrecv(intf, &req);
 
 	if (NULL != rsp) {
@@ -1539,7 +1661,7 @@ ipmi_sol_activate(struct ipmi_intf * intf, int looptest, int interval)
 
 
 	memcpy(&ap_rsp, rsp->data, sizeof(struct activate_payload_rsp));
-	
+
 	intf->session->sol_data.max_inbound_payload_size =
 		(ap_rsp.inbound_payload_size[1] << 8) |
 		ap_rsp.inbound_payload_size[0];
@@ -1551,7 +1673,6 @@ ipmi_sol_activate(struct ipmi_intf * intf, int looptest, int interval)
 	intf->session->sol_data.port =
 		(ap_rsp.payload_udp_port[1] << 8) |
 		ap_rsp.payload_udp_port[0];
-	
 
 
 	#if WORDS_BIGENDIAN
@@ -1564,7 +1685,7 @@ ipmi_sol_activate(struct ipmi_intf * intf, int looptest, int interval)
 	#endif
 
 
-	intf->session->timeout = 3;
+	intf->session->timeout = 1;
 
 
 	/* NOTE: the spec does allow for SOL traffic to be sent on
@@ -1582,7 +1703,6 @@ ipmi_sol_activate(struct ipmi_intf * intf, int looptest, int interval)
 			return -1;
 		}
 	}
-	
 
 	printf("[SOL Session operational.  Use %c? for help]\r\n",
 	       intf->session->sol_escape_char);
@@ -1592,7 +1712,7 @@ ipmi_sol_activate(struct ipmi_intf * intf, int looptest, int interval)
 		ipmi_sol_deactivate(intf);
 		usleep(interval*1000);
 		return 0;
-	}	
+	}
 
 	/*
 	 * At this point we are good to go with our SOL session.  We
@@ -1619,7 +1739,8 @@ print_sol_usage(void)
 {
 	lprintf(LOG_NOTICE, "SOL Commands: info [<channel number>]");
 	lprintf(LOG_NOTICE, "              set <parameter> <value> [channel]");
-	lprintf(LOG_NOTICE, "              activate");
+	lprintf(LOG_NOTICE, "              payload <enable|disable> [channel] [userid]");
+	lprintf(LOG_NOTICE, "              activate [<usesolforkeepalive|nokeepalive>]");
 	lprintf(LOG_NOTICE, "              deactivate");
 	lprintf(LOG_NOTICE, "              looptest [<loop times>] [<loop interval(in ms)>]");
 }
@@ -1678,49 +1799,104 @@ ipmi_sol_main(struct ipmi_intf * intf, int argc, char ** argv)
 			channel = (uint8_t)strtol(argv[1], NULL, 0);
 		else
 		{
-			print_sol_usage();	
+			print_sol_usage();
 			return -1;
 		}
 
 		retval = ipmi_print_sol_info(intf, channel);
 	}
 
+	/*
+	 * Payload enable or disable
+	 */
+	else if (!strncmp(argv[0], "payload", 7)) {
+		uint8_t channel = 0xe;
+		uint8_t userid = 1;
+		int enable = -1;
+
+		if (!strncmp(argv[1], "enable", 6))
+		{
+			enable = 1;
+		}
+		else if (!strncmp(argv[1], "disable", 7))
+		{
+			enable = 0;
+		}
+		else
+		{
+			print_sol_usage();
+			return -1;
+		}
+
+		if (argc >= 3)
+		{
+			channel = (uint8_t)strtol(argv[2], NULL, 0);
+		}
+		if (argc >= 4)
+		{
+			userid = (uint8_t)strtol(argv[3], NULL, 0);
+		}
+
+		retval = ipmi_sol_payload_access(intf, channel, userid, enable);
+	}
+
 
 	/*
 	 * Set a parameter value
 	 */
 	else if (!strncmp(argv[0], "set", 3)) {
-		uint8_t channel;
+		uint8_t channel = 0xe;
+		uint8_t guard = 1;
 
 		if (argc == 3)
-			channel = 0x0E; /* Ask about the current channel */
+		{
+			channel = 0xe;
+		}
 		else if (argc == 4)
+		{
+			if (!strncmp(argv[3], "noguard", 7))
+				guard = 0;
+			else
+				channel = (uint8_t)strtol(argv[3], NULL, 0);
+		}
+		else if (argc == 5)
+		{
 			channel = (uint8_t)strtol(argv[3], NULL, 0);
+			if (!strncmp(argv[4], "noguard", 7))
+				guard = 0;
+		}
 		else
 		{
 			print_sol_set_usage();
 			return -1;
 		}
-			
-		retval = ipmi_sol_set_param(intf,
-									channel,
-									argv[1],
-									argv[2]);
+
+		retval = ipmi_sol_set_param(intf, channel, argv[1], argv[2], guard);
 	}
-	
+
 
 	/*
 	 * Activate
 	 */
- 	else if (!strncmp(argv[0], "activate", 8))
+ 	else if (!strncmp(argv[0], "activate", 8)) {
+
+		if (argc > 1) {
+			if (!strncmp(argv[1], "usesolkeepalive", 11))
+				_use_sol_for_keepalive = 1;
+			else if (!strncmp(argv[1], "nokeepalive", 11))
+				_disable_keepalive = 1;
+		}
+
 		retval = ipmi_sol_activate(intf, 0, 0);
+	}
+
 
 	/*
 	 * Dectivate
 	 */
 	else if (!strncmp(argv[0], "deactivate", 10))
 		retval = ipmi_sol_deactivate(intf);
-	
+
 
 	/*
 	 * SOL loop test: Activate and then Dectivate
@@ -1730,9 +1906,9 @@ ipmi_sol_main(struct ipmi_intf * intf, int argc, char ** argv)
 		int cnt = 200;
 		int interval = 100; /* Unit is: ms */
 
-		if(argc > 3)
+		if (argc > 3)
 		{
-			print_sol_usage();	
+			print_sol_usage();
 			return -1;
 		}
 		if (argc != 1) /* at least 2 */
@@ -1745,12 +1921,12 @@ ipmi_sol_main(struct ipmi_intf * intf, int argc, char ** argv)
 			interval = strtol(argv[2], NULL, 10);
 			if(interval < 0) interval = 0;
 		}
-		
-		while(cnt > 0)
+
+		while (cnt > 0)
 		{
 			printf("remain loop test counter: %d\n", cnt);
 			retval = ipmi_sol_activate(intf, 1, interval);
-			if(retval)
+			if (retval)
 			{
 				printf("SOL looptest failed: %d\n", retval);
 				break;
