@@ -44,10 +44,29 @@ const struct valstr ipmi_oem_info[] = {
    { IPMI_OEM_TYAN,       "Tyan Computer Corporation" },
    { IPMI_OEM_NEWISYS,    "Newisys" },
    { IPMI_OEM_SUPERMICRO, "Supermicro" },
+   { IPMI_OEM_GOOGLE,     "Google" },
    { IPMI_OEM_KONTRON,    "Kontron" },
+   { IPMI_OEM_NOKIA,      "Nokia" },
    { 0xffff , NULL },
 };
 
+const struct oemvalstr ipmi_oem_sdr_type_vals[] = {
+   /* Keep OEM grouped together */
+   { IPMI_OEM_KONTRON , 0xC0 , "OEM Firmware Info" },
+   { IPMI_OEM_KONTRON , 0xC2 , "OEM Init Agent" },
+   { IPMI_OEM_KONTRON , 0xC3 , "OEM IPMBL Link State" },
+   { IPMI_OEM_KONTRON , 0xC4 , "OEM Board Reset" },
+   { IPMI_OEM_KONTRON , 0xC5 , "OEM FRU Information Agent" },
+   { IPMI_OEM_KONTRON , 0xC6 , "OEM POST Value Sensor" },
+   { IPMI_OEM_KONTRON , 0xC7 , "OEM FWUM Status" },
+   { IPMI_OEM_KONTRON , 0xC8 , "OEM Switch Mngt Software Status" },
+
+   { IPMI_OEM_PICMG   , 0xF0 , "PICMG FRU Hotswap" },
+   { IPMI_OEM_PICMG   , 0xF1 , "PICMG IPMB0 Link State" },
+
+   { 0xffff,            0x00,  NULL }
+};
+
 const struct valstr ipmi_netfn_vals[] = {
 	{ IPMI_NETFN_CHASSIS,	"Chassis" },
 	{ IPMI_NETFN_BRIDGE,	"Bridge" },
@@ -161,18 +180,26 @@ const struct valstr entity_id_vals[] = {
 	{ 0x26, "Remote Management Device" },
 	{ 0x27, "External Environment" },
 	{ 0x28, "Battery" },
-    { 0x29, "Processing Blade" },
-    { 0x2A, "Connectivity Switch" },
-    { 0x2B, "Processor/Memory Module" },
-    { 0x2C, "I/O Module" },
-    { 0x2D, "Processor/IO Module" },
-    { 0x2E, "Management Controller Firmware" },
-    { 0x2F, "IPMI Channel" },
-    { 0x30, "PCI Bus" },
-    { 0x31, "PCI Express Bus" },
-    { 0x32, "SCSI Bus (parallel)" },
-    { 0x33, "SATA/SAS Bus" },
-    { 0x34, "Processor/Front-Side Bus" },
+   { 0x29, "Processing Blade" },
+   { 0x2A, "Connectivity Switch" },
+   { 0x2B, "Processor/Memory Module" },
+   { 0x2C, "I/O Module" },
+   { 0x2D, "Processor/IO Module" },
+   { 0x2E, "Management Controller Firmware" },
+   { 0x2F, "IPMI Channel" },
+   { 0x30, "PCI Bus" },
+   { 0x31, "PCI Express Bus" },
+   { 0x32, "SCSI Bus (parallel)" },
+   { 0x33, "SATA/SAS Bus" },
+   { 0x34, "Processor/Front-Side Bus" },
+   /* PICMG */
+   { 0xA0, "PICMG Front Board" },
+   { 0xC0, "PICMG Rear Transition Module" },
+   { 0xC1, "PICMG AdvancedMC Module" },
+   { 0xF0, "PICMG Shelf Management Controller" },
+   { 0xF1, "PICMG Filtration Unit" },
+   { 0xF2, "PICMG Shelf FRU Information" },
+   { 0xF3, "PICMG Alarm Panel" },
 	{ 0x00, NULL },
 };
 
