@@ -49,13 +49,15 @@
 #include <sys/time.h>
 #include <sys/ioctl.h>
 
-#include <config.h>
+#if defined(HAVE_CONFIG_H)
+# include <config.h>
+#endif
 
 #if defined(HAVE_TERMIOS_H)
 # include <termios.h>
 #elif defined (HAVE_SYS_TERMIOS_H)
 # include <sys/termios.h>
-#endif        
+#endif
 
 #include <ipmitool/log.h>
 #include <ipmitool/helper.h>
@@ -71,6 +73,8 @@ static struct winsize _saved_winsize;
 static int _in_raw_mode = 0;
 static int _altterm = 0;
 
+extern int verbose;
+
 static int
 ipmi_tsol_command(struct ipmi_intf * intf, char *recvip, int port, unsigned char cmd)
 {
@@ -78,31 +82,25 @@ ipmi_tsol_command(struct ipmi_intf * intf, char *recvip, int port, unsigned char
         struct ipmi_rq	req;
         unsigned char	data[6];
 	unsigned	ip1, ip2, ip3, ip4;
-	uint16_t	portin;
-
-#if WORDS_BIGENDIAN
-	portin = BSWAP_16(port);
-#else
-	portin = port;
-#endif
 
         if (sscanf(recvip, "%d.%d.%d.%d", &ip1, &ip2, &ip3, &ip4) != 4) {
                 lprintf(LOG_ERR, "Invalid IP address: %s", recvip);
                 return -1;
         }
 
+	memset(&req, 0, sizeof(struct ipmi_rq));
         req.msg.netfn    = IPMI_NETFN_TSOL;
         req.msg.cmd      = cmd;
         req.msg.data_len = 6;
         req.msg.data     = data;
 
 	memset(data, 0, sizeof(data));
-        data[0] = ip1;  
-        data[1] = ip2;                    
+        data[0] = ip1;
+        data[1] = ip2;
         data[2] = ip3;
         data[3] = ip4;
-        data[4] = (portin & 0xff00) >> 8;
-        data[5] = (portin & 0xff);
+        data[4] = (port & 0xff00) >> 8;
+        data[5] = (port & 0xff);
 
         rsp = intf->sendrecv(intf, &req);
         if (rsp == NULL) {
@@ -130,14 +128,15 @@ ipmi_tsol_stop(struct ipmi_intf * intf, char *recvip, int port)
         return ipmi_tsol_command(intf, recvip, port, IPMI_TSOL_CMD_STOP);
 }
 
-static int                      
+static int
 ipmi_tsol_send_keystroke(struct ipmi_intf * intf, char *buff, int length)
-{                               
+{
         struct ipmi_rs * rsp;
         struct ipmi_rq   req;
         unsigned char    data[16];
 	static unsigned char keyseq = 0;
-                        
+
+	memset(&req, 0, sizeof(struct ipmi_rq));
         req.msg.netfn    = IPMI_NETFN_TSOL;
         req.msg.cmd      = IPMI_TSOL_CMD_SENDKEY;
         req.msg.data_len = length + 2;
@@ -147,26 +146,28 @@ ipmi_tsol_send_keystroke(struct ipmi_intf * intf, char *buff, int length)
         data[0] = length + 1;
 	memcpy(data + 1, buff, length);
 	data[length + 1] = keyseq++;
-                
+
         rsp = intf->sendrecv(intf, &req);
-        if (rsp == NULL) {
-		lprintf(LOG_ERR, "Unable to send keystroke");
-                return -1;
+	if (verbose) {
+		if (rsp == NULL) {
+			lprintf(LOG_ERR, "Unable to send keystroke");
+			return -1;
+		}
+		if (rsp->ccode > 0) {
+			lprintf(LOG_ERR, "Unable to send keystroke: %s",
+				val2str(rsp->ccode, completion_code_vals));
+			return -1;
+		}
 	}
-	if (rsp->ccode > 0) {
-		lprintf(LOG_ERR, "Unable to send keystroke: %s",
-			val2str(rsp->ccode, completion_code_vals));
-		return -1;
-        }
 
         return length;
-} 
+}
 
 static int
 tsol_keepalive(struct ipmi_intf * intf)
 {
         struct timeval end;
-        
+
         gettimeofday(&end, 0);
 
         if (end.tv_sec - _start_keepalive.tv_sec <= 30)
@@ -360,7 +361,7 @@ static void
 print_tsol_usage(void)
 {
 	struct winsize winsize;
-	
+
 	lprintf(LOG_NOTICE, "Usage: tsol [recvip] [port=NUM] [ro|rw] [rows=NUM] [cols=NUM] [altterm]");
 	lprintf(LOG_NOTICE, "       recvip       Receiver IP Address             [default=local]");
 	lprintf(LOG_NOTICE, "       port=NUM     Receiver UDP Port               [default=%d]",
@@ -421,7 +422,7 @@ ipmi_tsol_main(struct ipmi_intf * intf, int argc, char ** argv)
 		}
 	}
 
-	/* create udp socket to receive the packet */	
+	/* create udp socket to receive the packet */
 	memset(&sin, 0, sizeof(sin));
 	sin.sin_family = AF_INET;
 	sin.sin_port = htons(port);
@@ -475,7 +476,7 @@ ipmi_tsol_main(struct ipmi_intf * intf, int argc, char ** argv)
 	enter_raw_mode();
 
 	/*
-	 * talk to smdc to start Console redirect - IP address and port as parameter 
+	 * talk to smdc to start Console redirect - IP address and port as parameter
 	 * ipmitool -I lan -H 192.168.168.227 -U Administrator raw 0x30 0x06 0xC0 0xA8 0xA8 0x78 0x1A 0x0A
 	 */
 	result = ipmi_tsol_start(intf, recvip, port);
@@ -487,7 +488,7 @@ ipmi_tsol_main(struct ipmi_intf * intf, int argc, char ** argv)
 	printf("[SOL Session operational.  Use %c? for help]\r\n",
 	       intf->session->sol_escape_char);
 
-	gettimeofday(&_start_keepalive, 0);	
+	gettimeofday(&_start_keepalive, 0);
 
 	fds_wait[0].fd = fd_socket;
 	fds_wait[0].events = POLLIN;
@@ -498,7 +499,7 @@ ipmi_tsol_main(struct ipmi_intf * intf, int argc, char ** argv)
 	fds_wait[2].fd = -1;
 	fds_wait[2].events = 0;
 	fds_wait[2].revents = 0;
-	
+
 	fds_data_wait[0].fd = fd_socket;
 	fds_data_wait[0].events = POLLIN | POLLOUT;
 	fds_data_wait[0].revents = 0;
@@ -508,11 +509,11 @@ ipmi_tsol_main(struct ipmi_intf * intf, int argc, char ** argv)
 	fds_data_wait[2].fd = fileno(stdout);
 	fds_data_wait[2].events = POLLOUT;
 	fds_data_wait[2].revents = 0;
-	
+
 	out_buff_fill = 0;
 	in_buff_fill = 0;
 	fds = fds_wait;
-	
+
 	for (;;) {
 		result = poll(fds, 3, 15*1000);
 		if (result < 0)
@@ -551,7 +552,7 @@ ipmi_tsol_main(struct ipmi_intf * intf, int argc, char ** argv)
 					do_terminal_cleanup();
 					return result;
 				}
-				if (read_only) 
+				if (read_only)
 					bytes = 0;
 				in_buff_fill += bytes;
 			}
@@ -567,7 +568,7 @@ ipmi_tsol_main(struct ipmi_intf * intf, int argc, char ** argv)
 		}
 		if ((fds[0].revents & POLLOUT) && in_buff_fill) {
 			/*
-			 * translate key and send that to SMDC using IPMI 
+			 * translate key and send that to SMDC using IPMI
 			 * ipmitool -I lan -H 192.168.168.227 -U Administrator raw 0x30 0x03 0x04 0x1B 0x5B 0x43
 			 */
 			result = ipmi_tsol_send_keystroke(intf, in_buff, __min(in_buff_fill,14));
@@ -579,9 +580,9 @@ ipmi_tsol_main(struct ipmi_intf * intf, int argc, char ** argv)
 				}
 			}
 		}
-		fds = (in_buff_fill || out_buff_fill )? 
+		fds = (in_buff_fill || out_buff_fill )?
 			fds_data_wait : fds_wait;
 	}
-	
+
 	return 0;
 }
