@@ -58,6 +58,8 @@
 #include <ipmitool/ipmi_fwum.h>
 #include <ipmitool/ipmi_picmg.h>
 #include <ipmitool/ipmi_kontronoem.h>
+#include <ipmitool/ipmi_firewall.h>
+#include <ipmitool/ipmi_hpmfwupg.h>
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -77,6 +79,7 @@ int verbose = 0;
 struct ipmi_cmd ipmitool_cmd_list[] = {
 	{ ipmi_raw_main,     "raw",     "Send a RAW IPMI request and print response" },
 	{ ipmi_rawi2c_main,  "i2c",     "Send an I2C Master Write-Read command and print response" },
+	{ ipmi_rawspd_main,  "spd",     "Print SPD info from remote I2C device" },
 	{ ipmi_lanp_main,    "lan",     "Configure LAN Channels" },
 	{ ipmi_chassis_main, "chassis", "Get chassis status and set power state" },
 	{ ipmi_power_main,   "power",   "Shortcut to chassis power commands" },
@@ -98,12 +101,14 @@ struct ipmi_cmd ipmitool_cmd_list[] = {
 	{ ipmi_kontronoem_main, "kontronoem", "OEM Commands for Kontron devices"},
 	{ ipmi_picmg_main,   "picmg",   "Run a PICMG/ATCA extended cmd"},
 	{ ipmi_fwum_main,    "fwum",	"Update IPMC using Kontron OEM Firmware Update Manager" },
+	{ ipmi_firewall_main,"firewall","Configure Firmware Firewall" },
 #ifdef HAVE_READLINE
 	{ ipmi_shell_main,   "shell",   "Launch interactive IPMI shell" },
 #endif
 	{ ipmi_exec_main,    "exec",    "Run list of commands from file" },
 	{ ipmi_set_main,     "set",     "Set runtime variable for shell and exec" },
 	{ ipmi_echo_main,    "echo",    NULL }, /* for echoing lines to stdout in scripts */
+   { ipmi_hpmfwupg_main,"hpm", "Update HPM components using PICMG HPM.1 file"},
 	{ NULL },
 };
 
