@@ -59,6 +59,9 @@ extern struct ipmi_intf ipmi_lan_intf;
 #ifdef IPMI_INTF_LANPLUS
 extern struct ipmi_intf ipmi_lanplus_intf;
 #endif
+#ifdef IPMI_INTF_FREE
+extern struct ipmi_intf ipmi_free_intf;
+#endif
 
 struct ipmi_intf * ipmi_intf_table[] = {
 #ifdef IPMI_INTF_OPEN
@@ -78,6 +81,9 @@ struct ipmi_intf * ipmi_intf_table[] = {
 #endif
 #ifdef IPMI_INTF_LANPLUS
 	&ipmi_lanplus_intf,
+#endif
+#ifdef IPMI_INTF_FREE
+	&ipmi_free_intf,
 #endif
 	NULL
 };
@@ -212,6 +218,15 @@ ipmi_intf_session_set_privlvl(struct ipmi_intf * intf, uint8_t level)
 	intf->session->privlvl = level;
 }
 
+void
+ipmi_intf_session_set_lookupbit(struct ipmi_intf * intf, uint8_t lookupbit)
+{
+	if (intf->session == NULL)
+		return;
+
+	intf->session->v2_data.lookupbit = lookupbit;
+}
+
 void
 ipmi_intf_session_set_cipher_suite_id(struct ipmi_intf * intf, uint8_t cipher_suite_id)
 {
