@@ -40,9 +40,8 @@
 #include <arpa/inet.h>
 #include <errno.h>
 #include <unistd.h>
-#include <signal.h>
-#include <setjmp.h>
 #include <netdb.h>
+#include <time.h>
 #include <fcntl.h>
 #include <assert.h>
 
@@ -76,9 +75,6 @@ static struct ipmi_rq_entry * ipmi_req_entries;
 static struct ipmi_rq_entry * ipmi_req_entries_tail;
 
 
-static sigjmp_buf jmpbuf;
-
-
 static int ipmi_lanplus_setup(struct ipmi_intf * intf);
 static int ipmi_lanplus_keepalive(struct ipmi_intf * intf);
 static int ipmi_lan_send_packet(struct ipmi_intf * intf, uint8_t * data, int data_len);
@@ -265,13 +261,6 @@ void lanplus_swap(
 
 
 
-static void
-query_alarm(int signo)
-{
-	siglongjmp(jmpbuf, 1);
-}
-
-
 static const struct valstr plus_payload_types_vals[] = {
     { IPMI_PAYLOAD_TYPE_IPMI,              "IPMI (0)" },	// IPMI Message
     { IPMI_PAYLOAD_TYPE_SOL,               "SOL  (1)" },	// SOL (Serial over LAN)
@@ -302,7 +291,7 @@ ipmi_req_add_entry(struct ipmi_intf * intf, struct ipmi_rq * req, uint8_t req_se
 	memcpy(&e->req, req, sizeof(struct ipmi_rq));
 
 	e->intf = intf;
-   e->rq_seq = req_seq;
+	e->rq_seq = req_seq;
 
 	if (ipmi_req_entries == NULL)
 		ipmi_req_entries = e;
@@ -355,7 +344,7 @@ ipmi_req_remove_entry(uint8_t seq, uint8_t cmd)
 			else
 				ipmi_req_entries_tail = NULL;
 		}
-		
+
 		if (e->msg_data)
 			free(e->msg_data);
 		free(e);
@@ -384,7 +373,7 @@ ipmi_lan_send_packet(
 					 uint8_t * data, int
 					 data_len)
 {
-	if (verbose >= 2)
+	if (verbose >= 5)
 		printbuf(data, data_len, ">> sending packet");
 
 	return send(intf->fd, data, data_len, 0);
@@ -396,17 +385,22 @@ struct ipmi_rs *
 ipmi_lan_recv_packet(struct ipmi_intf * intf)
 {
 	static struct ipmi_rs rsp;
-	int rc;
+	fd_set read_set, err_set;
+	struct timeval tmout;
+	int ret;
 
-	/* setup alarm timeout */
-	if (sigsetjmp(jmpbuf, 1) != 0) {
-		alarm(0);
+	FD_ZERO(&read_set);
+	FD_SET(intf->fd, &read_set);
+
+	FD_ZERO(&err_set);
+	FD_SET(intf->fd, &err_set);
+
+	tmout.tv_sec = intf->session->timeout;
+	tmout.tv_usec = 0;
+
+	ret = select(intf->fd + 1, &read_set, NULL, &err_set, &tmout);
+	if (ret < 0 || FD_ISSET(intf->fd, &err_set) || !FD_ISSET(intf->fd, &read_set))
 		return NULL;
-	}
-
-	alarm(intf->session->timeout);
-	rc = recv(intf->fd, &rsp.data, IPMI_BUF_SIZE, 0);
-	alarm(0);
 
 	/* the first read may return ECONNREFUSED because the rmcp ping
 	 * packet--sent to UDP port 623--will be processed by both the
@@ -415,25 +409,40 @@ ipmi_lan_recv_packet(struct ipmi_intf * intf)
 	 * The problem with this is that the ECONNREFUSED takes
 	 * priority over any other received datagram; that means that
 	 * the Connection Refused shows up _before_ the response packet,
-	 * regardless of the order they were sent out.	(unless the
+	 * regardless of the order they were sent out.  (unless the
 	 * response is read before the connection refused is returned)
 	 */
-	if (rc < 0) {
-		alarm(intf->session->timeout);
-		rc = recv(intf->fd, &rsp.data, IPMI_BUF_SIZE, 0);
-		alarm(0);
-		if (rc < 0) {
-			perror("recv failed");
-			return NULL;
+	ret = recv(intf->fd, &rsp.data, IPMI_BUF_SIZE, 0);
+
+	if (ret < 0) {
+		FD_ZERO(&read_set);
+		FD_SET(intf->fd, &read_set);
+
+		FD_ZERO(&err_set);
+		FD_SET(intf->fd, &err_set);
+
+		tmout.tv_sec = intf->session->timeout;
+		tmout.tv_usec = 0;
+
+		ret = select(intf->fd + 1, &read_set, NULL, &err_set, &tmout);
+		if (ret < 0) {
+			if (FD_ISSET(intf->fd, &err_set) || !FD_ISSET(intf->fd, &read_set))
+				return NULL;
+
+			ret = recv(intf->fd, &rsp.data, IPMI_BUF_SIZE, 0);
+			if (ret < 0)
+				return NULL;
 		}
 	}
-	rsp.data[rc] = '\0';
-	rsp.data_len = rc;
 
-	if (verbose >= 2)
-	{
-		printbuf(rsp.data, rsp.data_len, "<< Received data");
-	}
+	if (ret == 0)
+		return NULL;
+
+	rsp.data[ret] = '\0';
+	rsp.data_len = ret;
+
+	if (verbose >= 5)
+		printbuf(rsp.data, rsp.data_len, "<< received packet");
 
 	return &rsp;
 }
@@ -581,16 +590,13 @@ ipmi_lan_poll_recv(struct ipmi_intf * intf)
 	struct rmcp_hdr rmcp_rsp;
 	struct ipmi_rs * rsp;
 	struct ipmi_session * session = intf->session;
-	
 	int offset, rv;
 	uint16_t payload_size;
+	uint8_t ourAddress = intf->my_addr;
 
-   uint8_t ourAddress = intf->my_addr;
-   
-   if(ourAddress == 0)
-   {
-      ourAddress = IPMI_BMC_SLAVE_ADDR;
-   }
+	if (ourAddress == 0) {
+		ourAddress = IPMI_BMC_SLAVE_ADDR;
+	}
 
 	rsp = ipmi_lan_recv_packet(intf);
 
@@ -608,7 +614,7 @@ ipmi_lan_poll_recv(struct ipmi_intf * intf)
 			rv = ipmi_handle_pong(intf, rsp);
 			return (rv <= 0) ? NULL : rsp;
 		}
-			
+
 		if (rmcp_rsp.class != RMCP_CLASS_IPMI) {
 			lprintf(LOG_DEBUG, "Invalid RMCP class: %x",
 				rmcp_rsp.class);
@@ -648,18 +654,17 @@ ipmi_lan_poll_recv(struct ipmi_intf * intf)
 			//continue;
 		}
 
-		
 		if ((session->v2_data.session_state == LANPLUS_STATE_ACTIVE)    &&
-			(rsp->session.authtype == IPMI_SESSION_AUTHTYPE_RMCP_PLUS) &&
-			(rsp->session.bEncrypted))
+		    (rsp->session.authtype == IPMI_SESSION_AUTHTYPE_RMCP_PLUS) &&
+		    (rsp->session.bEncrypted))
 
 		{
 			lanplus_decrypt_payload(session->v2_data.crypt_alg,
-									session->v2_data.k2,
-									rsp->data + offset,
-									rsp->session.msglen,
-									rsp->data + offset,
-									&payload_size);
+						session->v2_data.k2,
+						rsp->data + offset,
+						rsp->session.msglen,
+						rsp->data + offset,
+						&payload_size);
 		}
 		else
 			payload_size = rsp->session.msglen;
@@ -684,7 +689,6 @@ ipmi_lan_poll_recv(struct ipmi_intf * intf)
 				(long)rsp->session.id);
 			lprintf(LOG_DEBUG+1, "<<   Sequence                : 0x%08lx",
 				(long)rsp->session.seq);
-			
 			lprintf(LOG_DEBUG+1, "<<   IPMI Msg/Payload Length : %d",
 				rsp->session.msglen);
 			lprintf(LOG_DEBUG+1, "<< IPMI Response Message Header");
@@ -707,48 +711,40 @@ ipmi_lan_poll_recv(struct ipmi_intf * intf)
 
 			/* Are we expecting this packet? */
 			entry = ipmi_req_lookup_entry(rsp->payload.ipmi_response.rq_seq,
-										  rsp->payload.ipmi_response.cmd);
+						      rsp->payload.ipmi_response.cmd);
 			if (entry != NULL) {
 				lprintf(LOG_DEBUG+2, "IPMI Request Match found");
-            if (
-                  (intf->target_addr != ourAddress) &&
-                  (bridgePossible)
-               )
-            {
-				   if(
-                  (rsp->data_len) &&
-                  (rsp->payload.ipmi_response.cmd != 0x34)
-                 )
-				   {
-      			   printbuf(
-                           &rsp->data[offset], 
-                           (rsp->data_len-offset),
-                           "bridge command response");
-				   }
-				   /* bridged command: lose extra header */
-				   if (rsp->payload.ipmi_response.cmd == 0x34) {
-                  if( rsp->data_len == 38 )
-                  {
-					   entry->req.msg.cmd = entry->req.msg.target_cmd;
-					   rsp = ipmi_lan_recv_packet(intf);
-					   continue;
-                  }
-				   } else {
-					   //x += sizeof(rsp->payload.ipmi_response);
-					   if (rsp->data[offset-1] != 0)
-						   lprintf(LOG_DEBUG, "WARNING: Bridged "
-							   "cmd ccode = 0x%02x",
-						          rsp->data[offset-1]);
-				   }
-			   }
+				if (intf->target_addr != ourAddress &&
+				    bridgePossible) {
+					if (rsp->data_len &&
+					    rsp->payload.ipmi_response.cmd != 0x34) {
+						printbuf(
+							&rsp->data[offset],
+							(rsp->data_len-offset),
+							"bridge command response");
+					}
+					/* bridged command: lose extra header */
+					if (rsp->payload.ipmi_response.cmd == 0x34) {
+						if (rsp->data_len == 38) {
+							entry->req.msg.cmd = entry->req.msg.target_cmd;
+							rsp = ipmi_lan_recv_packet(intf);
+							continue;
+						}
+					} else {
+						//x += sizeof(rsp->payload.ipmi_response);
+						if (rsp->data[offset-1] != 0)
+							lprintf(LOG_DEBUG, "WARNING: Bridged "
+								"cmd ccode = 0x%02x",
+								rsp->data[offset-1]);
+					}
+				}
 				ipmi_req_remove_entry(rsp->payload.ipmi_response.rq_seq,
-									  rsp->payload.ipmi_response.cmd);
+						      rsp->payload.ipmi_response.cmd);
 			} else {
 				lprintf(LOG_INFO, "IPMI Request Match NOT FOUND");
 				rsp = ipmi_lan_recv_packet(intf);
 				continue;
-			}			
-
+			}
 
 			/*
 			 * Good packet.  Shift response data to start of array.
@@ -772,13 +768,13 @@ ipmi_lan_poll_recv(struct ipmi_intf * intf)
 		 * Open Response
 		 */
   		else if (rsp->session.payloadtype ==
-				 IPMI_PAYLOAD_TYPE_RMCP_OPEN_RESPONSE)
+			 IPMI_PAYLOAD_TYPE_RMCP_OPEN_RESPONSE)
 		{
 			if (session->v2_data.session_state !=
-				LANPLUS_STATE_OPEN_SESSION_SENT)
+			    LANPLUS_STATE_OPEN_SESSION_SENT)
 			{
 				lprintf(LOG_ERR, "Error: Received an Unexpected Open Session "
-					   "Response");
+					"Response");
 				rsp = ipmi_lan_recv_packet(intf);
 				continue;
 			}
@@ -786,7 +782,7 @@ ipmi_lan_poll_recv(struct ipmi_intf * intf)
 			read_open_session_response(rsp, offset);
 			break;
 		}
-		
+
 
 		/*
 		 * RAKP 2
@@ -821,7 +817,7 @@ ipmi_lan_poll_recv(struct ipmi_intf * intf)
 			break;
 		}
 
-		
+
 		/*
 		 * SOL
 		 */
@@ -853,7 +849,7 @@ ipmi_lan_poll_recv(struct ipmi_intf * intf)
 		else
 		{
 			lprintf(LOG_ERR, "Invalid RMCP+ payload type : 0x%x",
-				   rsp->session.payloadtype);
+				rsp->session.payloadtype);
 			assert(0);
 		}
 	}
@@ -1260,6 +1256,7 @@ void read_ipmi_response(struct ipmi_rs * rsp, int * offset)
  */
 void read_sol_packet(struct ipmi_rs * rsp, int * offset)
 {
+
 	/*
 	 * The data here should be decrypted by now.
 	 */
@@ -1278,38 +1275,36 @@ void read_sol_packet(struct ipmi_rs * rsp, int * offset)
 	rsp->payload.sol_packet.transfer_unavailable =
 		rsp->data[*offset] & 0x20;
 
-	rsp->payload.sol_packet.sol_inactive = 
+	rsp->payload.sol_packet.sol_inactive =
 		rsp->data[*offset] & 0x10;
 
 	rsp->payload.sol_packet.transmit_overrun =
 		rsp->data[*offset] & 0x08;
-	
+
 	rsp->payload.sol_packet.break_detected =
 		rsp->data[(*offset)++] & 0x04;
 
-	lprintf(LOG_DEBUG, "SOL sequence number     : 0x%02x",
+	lprintf(LOG_DEBUG, "<<<<<<<<<< RECV FROM BMC <<<<<<<<<<<");
+	lprintf(LOG_DEBUG, "< SOL sequence number     : 0x%02x",
 		rsp->payload.sol_packet.packet_sequence_number);
-
-	lprintf(LOG_DEBUG, "SOL acked packet        : 0x%02x",
+	lprintf(LOG_DEBUG, "< SOL acked packet        : 0x%02x",
 		rsp->payload.sol_packet.acked_packet_number);
-
-	lprintf(LOG_DEBUG, "SOL accepted char count : 0x%02x",
+	lprintf(LOG_DEBUG, "< SOL accepted char count : 0x%02x",
 		rsp->payload.sol_packet.accepted_character_count);
-
-	lprintf(LOG_DEBUG, "SOL is nack             : %s",
+	lprintf(LOG_DEBUG, "< SOL is nack             : %s",
 		rsp->payload.sol_packet.is_nack? "true" : "false");
-
-	lprintf(LOG_DEBUG, "SOL xfer unavailable    : %s",
+	lprintf(LOG_DEBUG, "< SOL xfer unavailable    : %s",
 		rsp->payload.sol_packet.transfer_unavailable? "true" : "false");
-		
-	lprintf(LOG_DEBUG, "SOL inactive            : %s",
+	lprintf(LOG_DEBUG, "< SOL inactive            : %s",
 		rsp->payload.sol_packet.sol_inactive? "true" : "false");
-
-	lprintf(LOG_DEBUG, "SOL transmit overrun    : %s",
+	lprintf(LOG_DEBUG, "< SOL transmit overrun    : %s",
 		rsp->payload.sol_packet.transmit_overrun? "true" : "false");
-
-	lprintf(LOG_DEBUG, "SOL break detected      : %s",
+	lprintf(LOG_DEBUG, "< SOL break detected      : %s",
 		rsp->payload.sol_packet.break_detected? "true" : "false");
+	lprintf(LOG_DEBUG, "<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<");
+
+	if (verbose >= 5)
+		printbuf(rsp->data + *offset - 4, 4, "SOL MSG FROM BMC");
 }
 
 
@@ -1422,10 +1417,32 @@ void getSolPayloadWireRep(
 {
 	int i = 0;
 
+	lprintf(LOG_DEBUG, ">>>>>>>>>> SENDING TO BMC >>>>>>>>>>");
+	lprintf(LOG_DEBUG, "> SOL sequence number     : 0x%02x",
+		payload->payload.sol_packet.packet_sequence_number);
+	lprintf(LOG_DEBUG, "> SOL acked packet        : 0x%02x",
+		payload->payload.sol_packet.acked_packet_number);
+	lprintf(LOG_DEBUG, "> SOL accepted char count : 0x%02x",
+		payload->payload.sol_packet.accepted_character_count);
+	lprintf(LOG_DEBUG, "> SOL is nack             : %s",
+		payload->payload.sol_packet.is_nack ? "true" : "false");
+	lprintf(LOG_DEBUG, "> SOL assert ring wor     : %s",
+		payload->payload.sol_packet.assert_ring_wor ? "true" : "false");
+	lprintf(LOG_DEBUG, "> SOL generate break      : %s",
+		payload->payload.sol_packet.generate_break ? "true" : "false");
+	lprintf(LOG_DEBUG, "> SOL deassert cts        : %s",
+		payload->payload.sol_packet.deassert_cts ? "true" : "false");
+	lprintf(LOG_DEBUG, "> SOL deassert dcd dsr    : %s",
+		payload->payload.sol_packet.deassert_dcd_dsr ? "true" : "false");
+	lprintf(LOG_DEBUG, "> SOL flush inbound       : %s",
+		payload->payload.sol_packet.flush_inbound ? "true" : "false");
+	lprintf(LOG_DEBUG, "> SOL flush outbound      : %s",
+		payload->payload.sol_packet.flush_outbound ? "true" : "false");
+
 	msg[i++] = payload->payload.sol_packet.packet_sequence_number;
 	msg[i++] = payload->payload.sol_packet.acked_packet_number;
 	msg[i++] = payload->payload.sol_packet.accepted_character_count;
-	
+
 	msg[i]    = payload->payload.sol_packet.is_nack           ? 0x40 : 0;
 	msg[i]   |= payload->payload.sol_packet.assert_ring_wor   ? 0x20 : 0;
 	msg[i]   |= payload->payload.sol_packet.generate_break    ? 0x10 : 0;
@@ -1439,6 +1456,13 @@ void getSolPayloadWireRep(
 		   payload->payload.sol_packet.data,
 		   payload->payload.sol_packet.character_count);
 
+	lprintf(LOG_DEBUG, "> SOL character count     : %d",
+		payload->payload.sol_packet.character_count);
+	lprintf(LOG_DEBUG, ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>");
+
+	if (verbose >= 5 && payload->payload.sol_packet.character_count)
+		printbuf(payload->payload.sol_packet.data, payload->payload.sol_packet.character_count, "SOL SEND DATA");
+
 	/*
 	 * At this point, the payload length becomes the whole payload
 	 * length, including the 4 bytes at the beginning of the SOL
@@ -1502,7 +1526,6 @@ ipmi_lanplus_build_v2x_msg(
 
 	/* msg will hold the entire message to be sent */
 	uint8_t * msg;
-
 	int len = 0;
 
 
@@ -1544,7 +1567,6 @@ ipmi_lanplus_build_v2x_msg(
 	/* Payload Type -- also specifies whether were authenticated/encyrpted */
 	msg[IMPI_LANPLUS_OFFSET_PAYLOAD_TYPE] = payload->payload_type;
 
-	
 	if (session->v2_data.session_state == LANPLUS_STATE_ACTIVE)
 	{
 		msg[IMPI_LANPLUS_OFFSET_PAYLOAD_TYPE] |=
@@ -1567,17 +1589,16 @@ ipmi_lanplus_build_v2x_msg(
 		msg[IMPI_LANPLUS_OFFSET_SEQUENCE_NUM + 2] = (session->out_seq >> 16) & 0xff;
 		msg[IMPI_LANPLUS_OFFSET_SEQUENCE_NUM + 3] = (session->out_seq >> 24) & 0xff;
 	}
-	
+
 	/*
 	 * Payload Length is set below (we don't know how big the payload is until after
 	 * encryption).
 	 */
-	
 
 	/*
 	 * Payload
 	 *
-	 * At this point we are ready to slam the payload in.  
+	 * At this point we are ready to slam the payload in.
 	 * This includes:
 	 * 1) The confidentiality header
 	 * 2) The payload proper (possibly encrypted)
@@ -1599,6 +1620,10 @@ ipmi_lanplus_build_v2x_msg(
 		getSolPayloadWireRep(intf,
 							 msg + IPMI_LANPLUS_OFFSET_PAYLOAD,
 							 payload);
+
+		if (verbose >= 5)
+			printbuf(msg + IPMI_LANPLUS_OFFSET_PAYLOAD, 4, "SOL MSG TO BMC");
+
 		len += payload->payload_length;
 
 		break;
@@ -1653,14 +1678,12 @@ ipmi_lanplus_build_v2x_msg(
 
 	}
 
-	
 	/* Now we know the payload length */
 	msg[IMPI_LANPLUS_OFFSET_PAYLOAD_SIZE    ] =
 		payload->payload_length        & 0xff;
 	msg[IMPI_LANPLUS_OFFSET_PAYLOAD_SIZE + 1] =
 		(payload->payload_length >> 8) & 0xff;
 
-	
 
 	/*
 	 *------------------------------------------
@@ -1697,11 +1720,10 @@ ipmi_lanplus_build_v2x_msg(
 
 		if (length_before_authcode % 4)
 			integrity_pad_size = 4 - (length_before_authcode % 4);
-							  
+
 		for (i = 0; i < integrity_pad_size; ++i)
 			msg[start_of_session_trailer + i] = 0xFF;
 
-		
 		/* Pad length */
 		msg[start_of_session_trailer + integrity_pad_size] = integrity_pad_size;
 
@@ -1709,7 +1731,6 @@ ipmi_lanplus_build_v2x_msg(
 		msg[start_of_session_trailer + integrity_pad_size + 1] =
 			0x07; /* Hardcoded per the spec, table 13-8 */
 
-		
 		hmac_input_size =
 			12                      +
 			payload->payload_length +
@@ -1784,9 +1805,9 @@ ipmi_lanplus_build_v2x_ipmi_cmd(
 	 * representation far below.
 	 */
  	static uint8_t curr_seq = 0;
-   
-   curr_seq += 1;
-   
+
+	curr_seq += 1;
+
 	if (curr_seq >= 64)
 		curr_seq = 0;
 
@@ -1799,12 +1820,12 @@ ipmi_lanplus_build_v2x_ipmi_cmd(
 	v2_payload.payload_length               = req->msg.data_len + 7;
 	v2_payload.payload.ipmi_request.request = req;
 	v2_payload.payload.ipmi_request.rq_seq  = curr_seq;
-	
-	ipmi_lanplus_build_v2x_msg(intf,                // in 
-							   &v2_payload,         // in 
-							   &(entry->msg_len),   // out
-							   &(entry->msg_data),  // out
-                        curr_seq); // in
+
+	ipmi_lanplus_build_v2x_msg(intf,                // in
+				   &v2_payload,         // in
+				   &(entry->msg_len),   // out
+				   &(entry->msg_data),  // out
+				   curr_seq); 		// in
 
 	return entry;
 }
@@ -1988,132 +2009,137 @@ ipmi_lanplus_send_payload(
 						  struct ipmi_v2_payload * payload)
 {
 	struct ipmi_rs      * rsp = NULL;
-	uint8_t       * msg_data;
+	uint8_t             * msg_data;
 	int                   msg_length;
 	struct ipmi_session * session = intf->session;
 	int                   try = 0;
+	int                   xmit = 1;
+	time_t                ltime;
 
 	if (!intf->opened && intf->open && intf->open(intf) < 0)
 		return NULL;
 
-	if (payload->payload_type == IPMI_PAYLOAD_TYPE_IPMI)
-	{
-		/*
-		 * Build an IPMI v1.5 or v2 command
-		 */
-		struct ipmi_rq_entry * entry;
-		struct ipmi_rq * ipmi_request = payload->payload.ipmi_request.request;
-
-		lprintf(LOG_DEBUG, "");
-		lprintf(LOG_DEBUG, ">> Sending IPMI command payload");
-		lprintf(LOG_DEBUG, ">>    netfn   : 0x%02x", ipmi_request->msg.netfn);
-		lprintf(LOG_DEBUG, ">>    command : 0x%02x", ipmi_request->msg.cmd);
-
-		if (verbose > 1)
-		{
-			uint16_t i;
-			fprintf(stderr, ">>    data    : ");
-			for (i = 0; i < ipmi_request->msg.data_len; ++i)
-				fprintf(stderr, "0x%02x ", ipmi_request->msg.data[i]);
-			fprintf(stderr, "\n\n");
-		}
-
-
-		/*
-		 * If we are presession, and the command is GET CHANNEL AUTHENTICATION
-		 * CAPABILITIES, we will build the command in v1.5 format.  This is so
-		 * that we can ask any server whether it supports IPMI v2 / RMCP+
-		 * before we attempt to open a v2.x session.
-		 */
-		if ((ipmi_request->msg.netfn == IPMI_NETFN_APP) &&
-			(ipmi_request->msg.cmd   == IPMI_GET_CHANNEL_AUTH_CAP) &&
-			(session->v2_data.bmc_id  == 0)) // jme - check
-		{
-			lprintf(LOG_DEBUG+1, "BUILDING A v1.5 COMMAND");
-			entry = ipmi_lanplus_build_v15_ipmi_cmd(intf, ipmi_request);
-		}
-		else
-		{
-			lprintf(LOG_DEBUG+1, "BUILDING A v2 COMMAND");
-			entry = ipmi_lanplus_build_v2x_ipmi_cmd(intf, ipmi_request);
-		}
-
-		if (entry == NULL) {
-			lprintf(LOG_ERR, "Aborting send command, unable to build");
-			return NULL;
-		}
-
-		msg_data   = entry->msg_data;
-		msg_length = entry->msg_len;
-	}
-
-	else if (payload->payload_type == IPMI_PAYLOAD_TYPE_RMCP_OPEN_REQUEST)
-	{
-		lprintf(LOG_DEBUG, ">> SENDING AN OPEN SESSION REQUEST\n");
-		assert(session->v2_data.session_state == LANPLUS_STATE_PRESESSION);
-
-		ipmi_lanplus_build_v2x_msg(intf,        /* in  */
-								   payload,     /* in  */
-								   &msg_length, /* out */
-								   &msg_data,   /* out */
-                           0);  /* irrelevant for this msg*/
-
-	}
-
-	else if (payload->payload_type == IPMI_PAYLOAD_TYPE_RAKP_1)
-	{
-		lprintf(LOG_DEBUG, ">> SENDING A RAKP 1 MESSAGE\n");
-		assert(session->v2_data.session_state ==
-			   LANPLUS_STATE_OPEN_SESSION_RECEIEVED);
-
-		ipmi_lanplus_build_v2x_msg(intf,        /* in  */
-								   payload,     /* in  */
-								   &msg_length, /* out */
-								   &msg_data,   /* out */
-                           0);  /* irrelevant for this msg*/
-
-	}
-
-	else if (payload->payload_type == IPMI_PAYLOAD_TYPE_RAKP_3)
-	{
-		lprintf(LOG_DEBUG, ">> SENDING A RAKP 3 MESSAGE\n");
-		assert(session->v2_data.session_state ==
-			   LANPLUS_STATE_RAKP_2_RECEIVED);
-
-		ipmi_lanplus_build_v2x_msg(intf,        /* in  */
-								   payload,     /* in  */
-								   &msg_length, /* out */
-								   &msg_data,   /* out */
-                           0);  /* irrelevant for this msg*/
-
-	}
-
-	else if (payload->payload_type == IPMI_PAYLOAD_TYPE_SOL)
-	{
-		lprintf(LOG_DEBUG, ">> SENDING A SOL MESSAGE\n");
-		assert(session->v2_data.session_state == LANPLUS_STATE_ACTIVE);
-
-		ipmi_lanplus_build_v2x_msg(intf,        /* in  */
-								   payload,     /* in  */
-								   &msg_length, /* out */
-								   &msg_data,   /* out */
-                           0);  /* irrelevant for this msg*/
-	}
-
-	else
-	{
-		lprintf(LOG_ERR, "Payload type 0x%0x is unsupported!",
-			payload->payload_type);
-		assert(0);
-	}
-
-
 	while (try < session->retry) {
+		ltime = time(NULL);
+
+		if (xmit) {
+
+			if (payload->payload_type == IPMI_PAYLOAD_TYPE_IPMI)
+			{
+				/*
+				 * Build an IPMI v1.5 or v2 command
+				 */
+				struct ipmi_rq_entry * entry;
+				struct ipmi_rq * ipmi_request = payload->payload.ipmi_request.request;
+
+				lprintf(LOG_DEBUG, "");
+				lprintf(LOG_DEBUG, ">> Sending IPMI command payload");
+				lprintf(LOG_DEBUG, ">>    netfn   : 0x%02x", ipmi_request->msg.netfn);
+				lprintf(LOG_DEBUG, ">>    command : 0x%02x", ipmi_request->msg.cmd);
+
+				if (verbose > 1)
+				{
+					uint16_t i;
+					fprintf(stderr, ">>    data    : ");
+					for (i = 0; i < ipmi_request->msg.data_len; ++i)
+						fprintf(stderr, "0x%02x ", ipmi_request->msg.data[i]);
+					fprintf(stderr, "\n\n");
+				}
 
 
-		if (ipmi_lan_send_packet(intf, msg_data, msg_length) < 0) {
-			lprintf(LOG_ERR, "IPMI LAN send command failed");
-			return NULL;
+				/*
+				 * If we are presession, and the command is GET CHANNEL AUTHENTICATION
+				 * CAPABILITIES, we will build the command in v1.5 format.  This is so
+				 * that we can ask any server whether it supports IPMI v2 / RMCP+
+				 * before we attempt to open a v2.x session.
+				 */
+				if ((ipmi_request->msg.netfn == IPMI_NETFN_APP) &&
+				    (ipmi_request->msg.cmd   == IPMI_GET_CHANNEL_AUTH_CAP) &&
+				    (session->v2_data.bmc_id  == 0)) // jme - check
+				{
+					lprintf(LOG_DEBUG+1, "BUILDING A v1.5 COMMAND");
+					entry = ipmi_lanplus_build_v15_ipmi_cmd(intf, ipmi_request);
+				}
+				else
+				{
+					lprintf(LOG_DEBUG+1, "BUILDING A v2 COMMAND");
+					entry = ipmi_lanplus_build_v2x_ipmi_cmd(intf, ipmi_request);
+				}
+
+				if (entry == NULL) {
+					lprintf(LOG_ERR, "Aborting send command, unable to build");
+					return NULL;
+				}
+
+				msg_data   = entry->msg_data;
+				msg_length = entry->msg_len;
+			}
+
+			else if (payload->payload_type == IPMI_PAYLOAD_TYPE_RMCP_OPEN_REQUEST)
+			{
+				lprintf(LOG_DEBUG, ">> SENDING AN OPEN SESSION REQUEST\n");
+				assert(session->v2_data.session_state == LANPLUS_STATE_PRESESSION);
+
+				ipmi_lanplus_build_v2x_msg(intf,        /* in  */
+							   payload,     /* in  */
+							   &msg_length, /* out */
+							   &msg_data,   /* out */
+							   0);  /* irrelevant for this msg*/
+
+			}
+
+			else if (payload->payload_type == IPMI_PAYLOAD_TYPE_RAKP_1)
+			{
+				lprintf(LOG_DEBUG, ">> SENDING A RAKP 1 MESSAGE\n");
+				assert(session->v2_data.session_state ==
+				       LANPLUS_STATE_OPEN_SESSION_RECEIEVED);
+
+				ipmi_lanplus_build_v2x_msg(intf,        /* in  */
+							   payload,     /* in  */
+							   &msg_length, /* out */
+							   &msg_data,   /* out */
+							   0);  /* irrelevant for this msg*/
+
+			}
+
+			else if (payload->payload_type == IPMI_PAYLOAD_TYPE_RAKP_3)
+			{
+				lprintf(LOG_DEBUG, ">> SENDING A RAKP 3 MESSAGE\n");
+				assert(session->v2_data.session_state ==
+				       LANPLUS_STATE_RAKP_2_RECEIVED);
+
+				ipmi_lanplus_build_v2x_msg(intf,        /* in  */
+							   payload,     /* in  */
+							   &msg_length, /* out */
+							   &msg_data,   /* out */
+							   0);  /* irrelevant for this msg*/
+
+			}
+
+			else if (payload->payload_type == IPMI_PAYLOAD_TYPE_SOL)
+			{
+				lprintf(LOG_DEBUG, ">> SENDING A SOL MESSAGE\n");
+				assert(session->v2_data.session_state == LANPLUS_STATE_ACTIVE);
+
+				ipmi_lanplus_build_v2x_msg(intf,        /* in  */
+							   payload,     /* in  */
+							   &msg_length, /* out */
+							   &msg_data,   /* out */
+							   0);  /* irrelevant for this msg*/
+			}
+
+			else
+			{
+				lprintf(LOG_ERR, "Payload type 0x%0x is unsupported!",
+					payload->payload_type);
+				assert(0);
+			}
+
+
+			if (ipmi_lan_send_packet(intf, msg_data, msg_length) < 0) {
+				lprintf(LOG_ERR, "IPMI LAN send command failed");
+				return NULL;
+			}
 		}
 
 		/* if we are set to noanswer we do not expect response */
@@ -2139,21 +2165,6 @@ ipmi_lanplus_send_payload(
 
 		/*
 		 * Special case for SOL outbound packets.
-		 *
-		 * Non-ACK packets require an ACK from the BMC (that matches
-		 * our packet!).
-		 *
-		 * While waiting for our ACK, it's very possible that we
-		 * will receive additional data form the BMC (that we will have
-		 * to ACK).
-		 *
-		 * Also, this is not perfectly correct.  We would like
-		 * to give the appropriate timeout for SOL retries, but our
-		 * timeout mechanism is in our recv() call.  Thus _any_
-		 * incoming packet from the BMC will use up one of our tries,
-		 * even if it is not our ACK, and even if it comes in before
-		 * our retry timeout.  I will make this code more sophisticated
-		 * if I see that this is a problem.
 		 */
 		if (payload->payload_type == IPMI_PAYLOAD_TYPE_SOL)
 		{
@@ -2169,7 +2180,6 @@ ipmi_lanplus_send_payload(
 			if (sol_response_acks_packet(rsp, payload))
 				break;
 
-			
 			else if (is_sol_packet(rsp) && rsp->data_len)
 			{
 				/*
@@ -2177,6 +2187,8 @@ ipmi_lanplus_send_payload(
 				 * the BMC
 				 */
 				intf->session->sol_data.sol_input_handler(rsp);
+            /* In order to avoid duplicate output, just set data_len to 0 */
+            rsp->data_len = 0;
 			}
 		}
 
@@ -2187,15 +2199,23 @@ ipmi_lanplus_send_payload(
 			rsp = ipmi_lan_poll_recv(intf);
 			if (rsp)
 				break;
-
 		}
 
+		xmit = ((time(NULL) - ltime) >= intf->session->timeout);
+
 		usleep(5000);
 
+		if (xmit) {
+			/* incremet session timeout each retry */
+			intf->session->timeout++;
+		}
+
 		try++;
 	}
 
-	
+	/* Reset timeout after retry loop completes */
+	intf->session->timeout = IPMI_LAN_TIMEOUT;
+
 	/* IPMI messages are deleted under ipmi_lan_poll_recv() */
 	switch (payload->payload_type) {
 	case IPMI_PAYLOAD_TYPE_RMCP_OPEN_REQUEST:
@@ -2220,6 +2240,7 @@ ipmi_lanplus_send_payload(
  *         0 if this isn't an ACK or we don't need to resend anything
  */
 int is_sol_partial_ack(
+					   struct ipmi_intf       * intf,
 					   struct ipmi_v2_payload * v2_payload,
 					   struct ipmi_rs         * rs)
 {
@@ -2232,6 +2253,10 @@ int is_sol_partial_ack(
 		(rs->payload.sol_packet.accepted_character_count <
 		 v2_payload->payload.sol_packet.character_count))
 	{
+		if (ipmi_oem_active(intf, "intelplus") &&
+		    rs->payload.sol_packet.accepted_character_count == 0)
+			return 0;
+
 		chars_to_resend =
 			v2_payload->payload.sol_packet.character_count -
 			rs->payload.sol_packet.accepted_character_count;
@@ -2292,16 +2317,17 @@ ipmi_lanplus_send_sol(
 	v2_payload->payload.sol_packet.acked_packet_number = 0; /* NA */
 
 	set_sol_packet_sequence_number(intf, v2_payload);
-	
+
 	v2_payload->payload.sol_packet.accepted_character_count = 0; /* NA */
 
 	rs = ipmi_lanplus_send_payload(intf, v2_payload);
 
 	/* Determine if we need to resend some of our data */
-	chars_to_resend = is_sol_partial_ack(v2_payload, rs);
+	chars_to_resend = is_sol_partial_ack(intf, v2_payload, rs);
 
-
-	while (chars_to_resend)
+	while (rs && !rs->payload.sol_packet.transfer_unavailable &&
+	       !rs->payload.sol_packet.is_nack &&
+	       chars_to_resend)
 	{
 		/*
 		 * We first need to handle any new data we might have
@@ -2311,7 +2337,7 @@ ipmi_lanplus_send_sol(
 			intf->session->sol_data.sol_input_handler(rs);
 
 		set_sol_packet_sequence_number(intf, v2_payload);
-		
+
 		/* Just send the required data */
 		memmove(v2_payload->payload.sol_packet.data,
 				v2_payload->payload.sol_packet.data +
@@ -2324,7 +2350,7 @@ ipmi_lanplus_send_sol(
 
 		rs = ipmi_lanplus_send_payload(intf, v2_payload);
 
-		chars_to_resend = is_sol_partial_ack(v2_payload, rs);
+		chars_to_resend = is_sol_partial_ack(intf, v2_payload, rs);
 	}
 
 	return rs;
@@ -2361,8 +2387,7 @@ check_sol_packet_for_new_data(
 	{
 		/* Store the data length before we mod it */
 		uint8_t unaltered_data_len = rsp->data_len;
-		
-		
+
 		if (rsp->payload.sol_packet.packet_sequence_number ==
 			last_received_sequence_number)
 		{
@@ -2438,7 +2463,7 @@ ack_sol_packet(
 			rsp->payload.sol_packet.packet_sequence_number;
 
 		ack.payload.sol_packet.accepted_character_count = rsp->data_len;
-		
+
 		ipmi_lanplus_send_payload(intf, &ack);
 	}
 }
@@ -2455,17 +2480,17 @@ struct ipmi_rs *
 ipmi_lanplus_recv_sol(struct ipmi_intf * intf)
 {
 	struct ipmi_rs * rsp = ipmi_lan_poll_recv(intf);
-	
-   if(rsp->session.authtype != 0)
-   {
-   	ack_sol_packet(intf, rsp);	              
 
-	   /*
-	    * Remembers the data sent, and alters the data to just
-	    * include the new stuff.
-	    */
-	   check_sol_packet_for_new_data(intf, rsp);
-   }
+	if (rsp && rsp->session.authtype != 0)
+	{
+		ack_sol_packet(intf, rsp);
+
+		/*
+		 * Remembers the data sent, and alters the data to just
+		 * include the new stuff.
+		 */
+		check_sol_packet_for_new_data(intf, rsp);
+	}
 	return rsp;
 }
 
@@ -2517,11 +2542,11 @@ ipmi_get_auth_capabilities_cmd(
 	struct ipmi_rs * rsp;
 	struct ipmi_rq req;
 	uint8_t msg_data[2];
-   uint8_t backupBridgePossible;
+	uint8_t backupBridgePossible;
 
-   backupBridgePossible = bridgePossible;
+	backupBridgePossible = bridgePossible;
 
-   bridgePossible = 0;
+	bridgePossible = 0;
 
 	msg_data[0] = IPMI_LAN_CHANNEL_E | 0x80; // Ask for IPMI v2 data as well
 	msg_data[1] = intf->session->privlvl;
@@ -2549,7 +2574,7 @@ ipmi_get_auth_capabilities_cmd(
 		}
 		if (rsp->ccode > 0) {
 			lprintf(LOG_INFO, "Get Auth Capabilities error: %s",
-				val2str(rsp->ccode, completion_code_vals));	
+				val2str(rsp->ccode, completion_code_vals));
 			return 1;
 		}
 	}
@@ -2559,7 +2584,7 @@ ipmi_get_auth_capabilities_cmd(
 		   rsp->data,
 		   sizeof(struct get_channel_auth_cap_rsp));
 
-   bridgePossible = backupBridgePossible;
+	bridgePossible = backupBridgePossible;
 
 	return 0;
 }
@@ -2573,20 +2598,20 @@ impi_close_session_cmd(struct ipmi_intf * intf)
 	struct ipmi_rq req;
 	uint8_t msg_data[4];
 	uint32_t bmc_session_lsbf;
-   uint8_t backupBridgePossible;
+	uint8_t backupBridgePossible;
 
 	if (intf->session->v2_data.session_state != LANPLUS_STATE_ACTIVE)
 		return -1;
 
-   backupBridgePossible = bridgePossible;
+	backupBridgePossible = bridgePossible;
 
-   bridgePossible = 0;
+	bridgePossible = 0;
 
 	bmc_session_lsbf = intf->session->v2_data.bmc_id;
-	#if WORDS_BIGENDIAN
+#if WORDS_BIGENDIAN
 	bmc_session_lsbf = BSWAP_32(bmc_session_lsbf);
-	#endif
-	
+#endif
+
 	memcpy(&msg_data, &bmc_session_lsbf, 4);
 
 	memset(&req, 0, sizeof(req));
@@ -2619,7 +2644,8 @@ impi_close_session_cmd(struct ipmi_intf * intf)
 	lprintf(LOG_DEBUG, "Closed Session %08lx\n",
 		(long)intf->session->v2_data.bmc_id);
 
-   bridgePossible = backupBridgePossible;
+	bridgePossible = backupBridgePossible;
+
 	return 0;
 }
 
@@ -2869,10 +2895,8 @@ ipmi_lanplus_rakp1(struct ipmi_intf * intf)
 	/*
 	 * Requested maximum privilege level.
 	 */
-	msg[24]  = 0x10; /* We will specify a name-only lookup */
-	msg[24] |= session->privlvl;
+	msg[24] = session->privlvl | session->v2_data.lookupbit;
 	session->v2_data.requested_role = msg[24];
-	
 	msg[25] = 0; /* reserved */
 	msg[26] = 0; /* reserved */
 
@@ -2905,7 +2929,7 @@ ipmi_lanplus_rakp1(struct ipmi_intf * intf)
 	}
 
 	session->v2_data.session_state = LANPLUS_STATE_RAKP_2_RECEIVED;
-	
+
 	if (verbose)
 		lanplus_dump_rakp2_message(rsp, session->v2_data.auth_alg);
 
@@ -3119,15 +3143,15 @@ ipmi_lanplus_close(struct ipmi_intf * intf)
 {
 	if (!intf->abort)
 		impi_close_session_cmd(intf);
-	
+
 	if (intf->fd >= 0)
 		close(intf->fd);
-	
+
 	ipmi_req_clear_entries();
-	
+
 	if (intf->session)
 		free(intf->session);
-	
+
 	intf->session = NULL;
 	intf->opened = 0;
 	intf = NULL;
@@ -3140,15 +3164,15 @@ ipmi_set_session_privlvl_cmd(struct ipmi_intf * intf)
 {
 	struct ipmi_rs * rsp;
 	struct ipmi_rq req;
-   uint8_t backupBridgePossible;
+	uint8_t backupBridgePossible;
 	uint8_t privlvl = intf->session->privlvl;
 
 	if (privlvl <= IPMI_SESSION_PRIV_USER)
 		return 0;	/* no need to set higher */
 
-   backupBridgePossible = bridgePossible;
+	backupBridgePossible = bridgePossible;
 
-   bridgePossible = 0;
+	bridgePossible = 0;
 
 	memset(&req, 0, sizeof(req));
 	req.msg.netfn		= IPMI_NETFN_APP;
@@ -3175,7 +3199,7 @@ ipmi_set_session_privlvl_cmd(struct ipmi_intf * intf)
 	lprintf(LOG_DEBUG, "Set Session Privilege Level to %s\n",
 		val2str(rsp->data[0], ipmi_privlvl_vals));
 
-   bridgePossible = backupBridgePossible;
+	bridgePossible = backupBridgePossible;
 
 	return 0;
 }
@@ -3187,7 +3211,6 @@ int
 ipmi_lanplus_open(struct ipmi_intf * intf)
 {
 	int rc;
-	struct sigaction act;
 	struct get_channel_auth_cap_rsp auth_cap;
 	struct sockaddr_in addr;
 	struct ipmi_session *session;
@@ -3265,16 +3288,6 @@ ipmi_lanplus_open(struct ipmi_intf * intf)
 		return -1;
 	}
 
-	/* setup alarm handler */
-	act.sa_handler = query_alarm;
-	act.sa_flags = 0;
-	if (sigemptyset(&act.sa_mask) == 0 &&
-	    sigaction(SIGALRM, &act, NULL) < 0) {
-		lperror(LOG_ERR, "alarm signal");
-		intf->close(intf);
-		return -1;
-	}
-
 	intf->opened = 1;
 
 
@@ -3329,8 +3342,8 @@ ipmi_lanplus_open(struct ipmi_intf * intf)
 
 	lprintf(LOG_DEBUG, "IPMIv2 / RMCP+ SESSION OPENED SUCCESSFULLY\n");
 
-   bridgePossible = 1;
-	
+	bridgePossible = 1;
+
 	rc = ipmi_set_session_privlvl_cmd(intf);
 	if (rc < 0)
 		goto fail;
@@ -3448,6 +3461,17 @@ ipmi_lanplus_keepalive(struct ipmi_intf * intf)
 		return 0;
 
 	rsp = intf->sendrecv(intf, &req);
+	while (rsp != NULL && is_sol_packet(rsp)) {
+                /* rsp was SOL data instead of our answer */
+                /* since it didn't go through the sol recv, do sol recv stuff here */
+                ack_sol_packet(intf, rsp);
+                check_sol_packet_for_new_data(intf, rsp);
+                if (rsp->data_len)
+                        intf->session->sol_data.sol_input_handler(rsp);
+		rsp = ipmi_lan_poll_recv(intf);
+		if (rsp == NULL) /* the get device id answer never got back, but retry mechanism was bypassed by SOL data */
+			return 0; /* so get device id command never returned, the connection is still alive */
+        }
 
 	if (rsp == NULL)
 		return -1;
